VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "BankAccount"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Option Explicit

Private m_cBalance As Currency

Public Function Deposit(Amount As Currency) As Boolean
'************************************************************************
'* Written By: Pierre G. Boutquin
'* Date:       January 31, 2000
'*
'* Syntax:     Deposit(Amount)
'*
'* Parameters:  Amount As Currency
'*
'* Purpose: This function will increase the account balance by the
'*          amount passed as a parameter. This amount must
'*          be positive. Returns True on Success, False otherwise.
'************************************************************************
   '--- Check the amount to make sure it's positive
   Deposit = (Amount >= 0)
   '--- If a good amount, add it to the balance
   If Deposit Then m_cBalance = m_cBalance + Amount
End Function

Public Function WithDdraw(Amount As Currency) As Boolean
'************************************************************************
'* Written By: Pierre G. Boutquin
'* Date:       January 31, 2000
'*
'* Syntax:     Deposit(Amount)
'*
'* Parameters:  Amount As Currency
'*
'* Purpose: This function will decrease the account balance by the
'*          amount passed as a parameter. This amount must
'*          be positive and there must be sufficient funds in
'*          the account to cover the amount.  Returns True on
'*          Success, False otherwise.
'************************************************************************
   '--- Check the withdraw amount to make sure it's available
   WithDdraw = (Amount >= 0) And (m_cBalance >= Amount)
   '--- If a valid amount, adjust the balance
   If WithDdraw Then m_cBalance = m_cBalance - Amount
End Function

Public Property Get Balance() As Currency
'************************************************************************
'* Written By: Pierre G. Boutquin
'* Date:       January 31, 2000
'*
'* Syntax:     Balance()
'*
'* Parameters:  Amount As Currency
'*
'* Purpose: This property will return the account balance.
'************************************************************************
   Balance = m_cBalance
End Property


Public Property Get IsClient() As Boolean
'************************************************************************
'* Written By: Pierre G. Boutquin
'* Date:       January 31, 2000
'*
'* Syntax:     IsClient()
'*
'* Parameters:  None.
'*
'* Purpose: This property will return TRUE if the client is a member
'*          of the ATMClient security role.
'************************************************************************
   Dim oSecurityCallContext As COMSVCSLib.SecurityCallContext

   '--- Get the security context
   Set oSecurityCallContext = COMSVCSLib.GetSecurityCallContext()

   '--- Initialize the return value
   IsClient = False

   '--- Check to see if security is enabled
   If oSecurityCallContext.IsSecurityEnabled() Then
      '--- Check to see if the caller is in the correct role
      '--- Adjust the return value if the caller is in the correct role
      If oSecurityCallContext.IsCallerInRole("ATMClient") Then IsClient = True
   End If
End Property

