VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCertRequest"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'
' Certificate Request Format
'
Const CR_IN_BASE64HEADER = 0
Const CR_IN_BASE64 = &H1
Const CR_IN_BINARY = &H2
Const CR_IN_ENCODEMASK = &HFF
Const CR_IN_PKCS10 = &H100
Const CR_IN_KEYGEN = &H200
Const CR_IN_FORMATMASK = &HFF00
Const CR_IN_ENCRYPTED_REQUEST = &H10000
Const CR_IN_ENCRYPTED_ATTRIBUTES = &H20000

'
' Certificate Request Status
'
Const CR_DISP_INCOMPLETE = 0
Const CR_DISP_ERROR = &H1
Const CR_DISP_DENIED = &H2
Const CR_DISP_ISSUED = &H3
Const CR_DISP_ISSUED_OUT_OF_BAND = &H4
Const CR_DISP_UNDER_SUBMISSION = &H5

'
' Issued Certificate Format
'
Const CR_OUT_BASE64HEADER = 0
Const CR_OUT_BASE64 = &H1
Const CR_OUT_BINARY = &H2
Const CR_OUT_ENCODEMASK = &HFF
Const CR_OUT_CHAIN = &H100

'
' Key Types
'
Const AT_KEYEXCHANGE = 1
Const AT_SIGNATURE = 2

Public Enum enmKeyType
    KT_KeyExchange = AT_KEYEXCHANGE
    KT_Signature = AT_SIGNATURE
End Enum

'+-------------------------------------------------------------------------
'  Enhanced Key Usage (Purpose) Object Identifiers
'--------------------------------------------------------------------------
Const szOID_PKIX_KP = "1.3.6.1.5.5.7.3"

' Consistent key usage bits: DIGITAL_SIGNATURE, KEY_ENCIPHERMENT
' or KEY_AGREEMENT
Const szOID_PKIX_KP_SERVER_AUTH = "1.3.6.1.5.5.7.3.1"

' Consistent key usage bits: DIGITAL_SIGNATURE
Const szOID_PKIX_KP_CLIENT_AUTH = "1.3.6.1.5.5.7.3.2"

' Consistent key usage bits: DIGITAL_SIGNATURE
Const szOID_PKIX_KP_CODE_SIGNING = "1.3.6.1.5.5.7.3.3"

' Consistent key usage bits: DIGITAL_SIGNATURE, NON_REPUDIATION and/or
' (KEY_ENCIPHERMENT or KEY_AGREEMENT)
Const szOID_PKIX_KP_EMAIL_PROTECTION = "1.3.6.1.5.5.7.3.4"

Public Enum enmKeyUsage
    KU_ClientAuthentication = 0
    KU_EMailProtection = 1
    KU_ServerAuthentication = 2
    KU_CodeSigning = 3
End Enum

'+-------------------------------------------------------------------------
'  Microsoft Enhanced Key Usage (Purpose) Object Identifiers
'+-------------------------------------------------------------------------

'  Signer of CTLs
Const szOID_KP_CTL_USAGE_SIGNING = "1.3.6.1.4.1.311.10.3.1"

'  Signer of TimeStamps
Const szOID_KP_TIME_STAMP_SIGNING = "1.3.6.1.4.1.311.10.3.2"

'+-------------------------------------------------------------------------
'  Microsoft Attribute Object Identifiers
'+-------------------------------------------------------------------------
Const szOID_YESNO_TRUST_ATTR = "1.3.6.1.4.1.311.10.4.1"

'"1.3.6.1.4.1.311.2.1.21")

'-- Properties
Private m_strUserName As String
Private m_strUnit As String
Private m_strOrganization As String
Private m_strCity As String
Private m_strState As String
Private m_strCountry As String
Private m_lReqID As Long
Private m_strServer As String
Private m_strCA As String
Private m_strEMail As String
Private m_lKeySpec As enmKeyType
Private m_strKeyUsage As String
Private m_lKeyUsage As enmKeyUsage

'-- Objects for Enrollment and Request controls
Private m_objXen As Object
Private m_objReq As CCertRequest

Public Function DoCertRequest() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 12, 1999
'*
'* Syntax:     DoCertRequest
'*
'* Parameters:  None
'*
'* Purpose: This function builds the distinguished name string
'*          and then calls the XEnroll control to generate a
'*          public/private key pair and export the public key
'*          in the form of a certificate request. The certificate
'*          request is then submitted to the Certificate Authority
'*          using the Certificate Request control.
'*************************************************************
    DoCertRequest = False
    
    Dim strDN As String
    Dim strReq As String
    Dim lDisp As Long
    Dim strMsg As String
    
    On Error GoTo DoCertReqErr
    
    '--- Build the distinguished name string
    strDN = "CN=" + m_strUserName _
        + ",OU=" + m_strUnit _
        + ",O=" + m_strOrganization _
        + ",L=" + m_strCity _
        + ",S=" + m_strState _
        + ",C=" + m_strCountry
        
    '--- Do we have an E-Mail address?
    If (m_strEMail <> "") Then
        '--- Yes, so include it also
        strDN = strDN + ",E=" + m_strEMail
    End If
        
    '--- Specify the key type
    m_objXen.KeySpec = m_lKeySpec
    
    '--- Create the PKCS #10 request
    strReq = m_objXen.createPKCS10(strDN, m_strKeyUsage)
    
    '--- Submit the request to the CA
    lDisp = m_objReq.Submit(CR_IN_BASE64 Or CR_IN_PKCS10, _
                            strReq, _
                            "", _
                            m_strServer + "\" + m_strCA)
    
    '--- Get the textual description of the request status
    strMsg = m_objReq.GetDispositionMessage
    '--- Display it for the user
    MsgBox strMsg, vbOKOnly, "Certificate Request"
    
    '--- Did we fail? If so, then exit early
    If ((lDisp = CR_DISP_INCOMPLETE) Or _
        (lDisp = CR_DISP_ERROR) Or _
        (lDisp = CR_DISP_DENIED)) Then Exit Function
    
    '--- Get the request ID
    m_lReqID = m_objReq.GetRequestId
    
    '--- If the certificate has already been issued, then get the certificate
    If (lDisp = CR_DISP_ISSUED) Or (lDisp = CR_DISP_ISSUED_OUT_OF_BAND) Then
        If Not GetCert Then Exit Function
    End If
    
    '--- Didn't exit early, return TRUE
    DoCertRequest = True
    Exit Function
    
DoCertReqErr:
    '--- Tell the user about the error
    MsgBox "Error - " + Str(Err.Number) + " - " + Err.Description
    
End Function

Public Property Get UserName() As String
    UserName = m_strUserName
End Property

Public Property Let UserName(ByVal sNewValue As String)
    m_strUserName = sNewValue
End Property

Public Property Get EMail() As String
    EMail = m_strEMail
End Property

Public Property Let EMail(ByVal sNewValue As String)
    m_strEMail = sNewValue
End Property

Public Property Get Unit() As String
    Unit = m_strUnit
End Property

Public Property Let Unit(ByVal sNewValue As String)
    m_strUnit = sNewValue
End Property

Public Property Get Organization() As String
    Organization = m_strOrganization
End Property

Public Property Let Organization(ByVal sNewValue As String)
    m_strOrganization = sNewValue
End Property

Public Property Get City() As String
    City = m_strCity
End Property

Public Property Let City(ByVal sNewValue As String)
    m_strCity = sNewValue
End Property

Public Property Get State() As String
    State = m_strState
End Property

Public Property Let State(ByVal sNewValue As String)
    m_strState = sNewValue
End Property

Public Property Get Country() As String
    Country = m_strCountry
End Property

Public Property Let Country(ByVal sNewValue As String)
    m_strCountry = sNewValue
End Property

Public Property Get RequestID() As Long
    RequestID = m_lReqID
End Property

Private Sub Class_Initialize()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 12, 1999
'*
'* Syntax:     Class_Initialize
'*
'* Parameters:  None
'*
'* Purpose: During the class initialization, the instances of
'*          the Certificate Enroll and Certificate Request
'*          objects are created.
'*************************************************************
    On Error GoTo InitErr
    
    m_lReqID = 0
    
    '--- Instantiate the certificate enrollment object
    Set m_objXen = CreateObject("CEnroll.CEnroll.1")
    
    '--- Instantiate the certificate request object
    Set m_objReq = New CCertRequest
    
    Exit Sub
InitErr:
    '--- Tell the user about the error
    MsgBox "Error creating request objects - " + Str(Err.Number) + " - " + Err.Description
End Sub

Private Sub Class_Terminate()
    '--- Free the objects
    Set m_objXen = Nothing
    Set m_objReq = Nothing
End Sub

Public Function GetReqStatus() As Long
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 12, 1999
'*
'* Syntax:     GetReqStatus
'*
'* Parameters:  None
'*
'* Purpose: This function checks the current status of the
'*          current outstanding certificate request. If the
'*          request has been issued, then the certificate is
'*          retrieved.
'*************************************************************
    Dim lDisp As Long
    Dim strMsg As String
    
    '--- Retrieve the current status of our request
    lDisp = m_objReq.RetrievePending(m_lReqID, m_strServer + "\" + m_strCA)
    
    '--- Get the textual description of the request status
    strMsg = m_objReq.GetDispositionMessage
    '--- Display it for the user
    MsgBox strMsg, vbOKOnly, "Certificate Request"
    
    '--- Set the status code to the return value
    GetReqStatus = lDisp
    
    '--- If the certificate has already been issued, then get the certificate
    If (lDisp = CR_DISP_ISSUED) Or (lDisp = CR_DISP_ISSUED_OUT_OF_BAND) Then
        If Not GetCert Then Exit Function
    End If
End Function

Public Property Get Server() As String
    Server = m_strServer
End Property

Public Property Let Server(ByVal sNewValue As String)
    m_strServer = sNewValue
End Property

Public Property Get CertificateAuthority() As String
    CertificateAuthority = m_strCA
End Property

Public Property Let CertificateAuthority(ByVal sNewValue As String)
    m_strCA = sNewValue
End Property

Public Function GetCert() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 12, 1999
'*
'* Syntax:     GetCert
'*
'* Parameters:  None
'*
'* Purpose: This function retrieves the issued certificate from
'*          the CA using the Certificate Request control. Next,
'*          the Certificate Enroll control is used to accept the
'*          certificate and to place it into the certificate
'*          store.
'*************************************************************
    On Error GoTo GetCertErr
    Dim strCert As String
    
    '--- Initialize the return value
    GetCert = False
    
    '--- Get the certificate
    strCert = m_objReq.GetCertificate(CR_OUT_BINARY Or CR_OUT_CHAIN)
    
    '--- Accept the certificate into the certificate store
    m_objXen.acceptPKCS7 strCert
    
    '--- Didn't exit early, return TRUE
    GetCert = True
    
    Exit Function
    
GetCertErr:
    '--- Tell the user about the error
    MsgBox "Error - " + Str(Err.Number) + " - " + Err.Description
End Function

Public Property Get KeySpec() As enmKeyType
    KeySpec = m_lKeySpec
End Property

Public Property Let KeySpec(ByVal lNewValue As enmKeyType)
    m_lKeySpec = lNewValue
End Property

Public Property Get KeyUsage() As enmKeyUsage
    KeyUsage = m_lKeyUsage
End Property

Public Property Let KeyUsage(ByVal lNewValue As enmKeyUsage)
    '--- Set the key usage value
    m_lKeyUsage = lNewValue
    
    '--- Set the Key Usage OID to the appropriate string
    Select Case m_lKeyUsage
        Case KU_ClientAuthentication
            m_strKeyUsage = szOID_PKIX_KP_CLIENT_AUTH
        Case KU_EMailProtection
            m_strKeyUsage = szOID_PKIX_KP_EMAIL_PROTECTION
        Case KU_ServerAuthentication
            m_strKeyUsage = szOID_PKIX_KP_SERVER_AUTH
        Case KU_CodeSigning
            m_strKeyUsage = szOID_PKIX_KP_CODE_SIGNING
    End Select
End Property
