VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCrypto"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Purpose: This class encapsulates a small amount of the
'*          Microsoft CryptoAPI functionality. As of the
'*          initial creation, it provides functionality to
'*          exchange public keys and pass messages using the
'*          AT_KEYEXCHANGE keys only. A session key is
'*          generated for each message encrypted, so the
'*          initial version of this class is not suited for
'*          encrypting large amounts of data on a continuous
'*          basis. To support that capability, this class
'*          would need to be enhanced to include a class-level
'*          session key that was maintained and used for all
'*          encrypting and decrypting.
'*
'* Revisions
'*   Name          Date     Description
'*   ---------------------------------------------------------
'*************************************************************
'+---------------------------------------------------------------------------
'
'  Microsoft Windows
'  Copyright 1992 - 1998 Microsoft Corporation.
'
'  File:       wincrypt.h
'
'  Contents:   Cryptographic API Prototypes and Definitions
'
'----------------------------------------------------------------------------


'
' Algorithm IDs and Flags
'

' Algorithm classes
Private Const ALG_CLASS_ANY = 0
Private Const ALG_CLASS_SIGNATURE = 8192
Private Const ALG_CLASS_MSG_ENCRYPT = 16384
Private Const ALG_CLASS_DATA_ENCRYPT = 24576
Private Const ALG_CLASS_HASH = 32768
Private Const ALG_CLASS_KEY_EXCHANGE = 40960

' Algorithm types
Private Const ALG_TYPE_ANY = 0
Private Const ALG_TYPE_DSS = 512
Private Const ALG_TYPE_RSA = 1024
Private Const ALG_TYPE_BLOCK = 1536
Private Const ALG_TYPE_STREAM = 2048
Private Const ALG_TYPE_DH = 2560
Private Const ALG_TYPE_SECURECHANNEL = 3072

' RC2 sub-ids
Private Const ALG_SID_RC2 = 2

' Stream cipher sub-ids
Private Const ALG_SID_RC4 = 1
Private Const ALG_SID_SEAL = 2

' Diffie-Hellman sub-ids
Private Const ALG_SID_DH_SANDF = 1
Private Const ALG_SID_DH_EPHEM = 2
Private Const ALG_SID_AGREED_KEY_ANY = 3
Private Const ALG_SID_KEA = 4

' Hash sub ids
Private Const ALG_SID_MD2 = 1
Private Const ALG_SID_MD4 = 2
Private Const ALG_SID_MD5 = 3
Private Const ALG_SID_SHA = 4
Private Const ALG_SID_SHA1 = 4
Private Const ALG_SID_MAC = 5
Private Const ALG_SID_RIPEMD = 6
Private Const ALG_SID_RIPEMD160 = 7
Private Const ALG_SID_SSL3SHAMD5 = 8
Private Const ALG_SID_HMAC = 9

' algorithm identifier definitions
Private Const CALG_MD5 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MD5)
Private Const CALG_RC2 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_RC2)
Private Const CALG_SSL3_SHAMD5 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_SSL3SHAMD5)

' dwFlags definitions for CryptAcquireContext
Private Const CRYPT_VERIFYCONTEXT = &HF0000000
Private Const CRYPT_NEWKEYSET = &H8
Private Const CRYPT_DELETEKEYSET = &H10
Private Const CRYPT_MACHINE_KEYSET = &H20

' dwFlag definitions for CryptGenKey
Private Const CRYPT_EXPORTABLE = &H1
Private Const CRYPT_USER_PROTECTED = &H2
Private Const CRYPT_CREATE_SALT = &H4
Private Const CRYPT_UPDATE_KEY = &H8
Private Const CRYPT_NO_SALT = &H10
Private Const CRYPT_PREGEN = &H40
Private Const CRYPT_RECIPIENT = &H10
Private Const CRYPT_INITIATOR = &H40
Private Const CRYPT_ONLINE = &H80
Private Const CRYPT_SF = &H100
Private Const CRYPT_CREATE_IV = &H200
Private Const CRYPT_KEK = &H400
Private Const CRYPT_DATA_KEY = &H800

' dwFlags definitions for CryptDeriveKey
Private Const CRYPT_SERVER = &H400

Private Const KEY_LENGTH_MASK = &HFFFF0000

' dwFlag definitions for CryptSetProviderEx and CryptGetDefaultProvider
Private Const CRYPT_MACHINE_DEFAULT = &H1
Private Const CRYPT_USER_DEFAULT = &H2
Private Const CRYPT_DELETE_DEFAULT = &H4

' exported key blob definitions
Private Const SIMPLEBLOB = &H1
Private Const PUBLICKEYBLOB = &H6
Private Const PRIVATEKEYBLOB = &H7
Private Const PLAINTEXTKEYBLOB = &H8

Private Const AT_KEYEXCHANGE = 1
Private Const AT_SIGNATURE = 2

Private Const HP_ALGID = &H1                     ' Hash algorithm
Private Const HP_HASHVAL = &H2                   ' Hash value
Private Const HP_HASHSIZE = &H4                  ' Hash value size
Private Const HP_HMAC_INFO = &H5                 ' information for creating an HMAC

Private Const CRYPT_FAILED = 0
Private Const CRYPT_SUCCEED = -1

Private Const PROV_RSA_FULL = 1
Private Const PROV_RSA_SIG = 2
Private Const PROV_DSS = 3
Private Const PROV_FORTEZZA = 4
Private Const PROV_MS_EXCHANGE = 5
Private Const PROV_SSL = 6
Private Const PROV_RSA_SCHANNEL = 12
Private Const PROV_DSS_DH = 13
Private Const PROV_EC_ECDSA_SIG = 14
Private Const PROV_EC_ECNRA_SIG = 15
Private Const PROV_EC_ECDSA_FULL = 16
Private Const PROV_EC_ECNRA_FULL = 17
Private Const PROV_SPYRUS_LYNKS = 20

'
' Provider friendly names
'
Private Const MS_DEF_PROV = "Microsoft Base Cryptographic Provider v1.0"
Private Const MS_ENHANCED_PROV = "Microsoft Enhanced Cryptographic Provider v1.0"
Private Const MS_DEF_RSA_SIG_PROV = "Microsoft RSA Signature Cryptographic Provider"
Private Const MS_DEF_RSA_SCHANNEL_PROV = "Microsoft Base RSA SChannel Cryptographic Provider"
Private Const MS_ENHANCED_RSA_SCHANNEL_PROV = "Microsoft Enhanced RSA SChannel Cryptographic Provider"
Private Const MS_DEF_DSS_PROV = "Microsoft Base DSS Cryptographic Provider"
Private Const MS_DEF_DSS_DH_PROV = "Microsoft Base DSS and Diffie-Hellman Cryptographic Provider"

'+-------------------------------------------------------------------------
'  Certificate name types
'--------------------------------------------------------------------------
Private Const CERT_NAME_EMAIL_TYPE = 1
Private Const CERT_NAME_RDN_TYPE = 2
Private Const CERT_NAME_ATTR_TYPE = 3
Private Const CERT_NAME_SIMPLE_DISPLAY_TYPE = 4
Private Const CERT_NAME_FRIENDLY_DISPLAY_TYPE = 5

Private Const CERT_SYSTEM_STORE_MASK = &HFFFF0000

'+-------------------------------------------------------------------------
'  Certificate, CRL and CTL property IDs
'
'  See CertSetCertificateContextProperty or CertGetCertificateContextProperty
'  for usage information.
'--------------------------------------------------------------------------
Private Const CERT_KEY_PROV_HANDLE_PROP_ID = 1
Private Const CERT_KEY_PROV_INFO_PROP_ID = 2
Private Const CERT_SHA1_HASH_PROP_ID = 3
Private Const CERT_MD5_HASH_PROP_ID = 4
Private Const CERT_HASH_PROP_ID = CERT_SHA1_HASH_PROP_ID
Private Const CERT_KEY_CONTEXT_PROP_ID = 5
Private Const CERT_KEY_SPEC_PROP_ID = 6
Private Const CERT_IE30_RESERVED_PROP_ID = 7
Private Const CERT_PUBKEY_HASH_RESERVED_PROP_ID = 8
Private Const CERT_ENHKEY_USAGE_PROP_ID = 9
Private Const CERT_CTL_USAGE_PROP_ID = CERT_ENHKEY_USAGE_PROP_ID
Private Const CERT_NEXT_UPDATE_LOCATION_PROP_ID = 10
Private Const CERT_FRIENDLY_NAME_PROP_ID = 11
Private Const CERT_PVK_FILE_PROP_ID = 12
Private Const CERT_DESCRIPTION_PROP_ID = 13
Private Const CERT_ACCESS_STATE_PROP_ID = 14
Private Const CERT_SIGNATURE_HASH_PROP_ID = 15
Private Const CERT_SMART_CARD_DATA_PROP_ID = 16
Private Const CERT_EFS_PROP_ID = 17
Private Const CERT_FORTEZZA_DATA_PROP_ID = 18
Private Const CERT_ARCHIVED_PROP_ID = 19
Private Const CERT_KEY_IDENTIFIER_PROP_ID = 20
Private Const CERT_AUTO_ENROLL_PROP_ID = 21
Private Const CERT_PUBKEY_ALG_PARA_PROP_ID = 22

Private Const CERT_FIRST_RESERVED_PROP_ID = 23
' Note, 32 - 35 are reserved for the CERT, CRL, CTL and KeyId file element IDs.

Private Const CERT_LAST_RESERVED_PROP_ID = &H7FFF
Private Const CERT_FIRST_USER_PROP_ID = &H8000
Private Const CERT_LAST_USER_PROP_ID = &HFFFF

Private Type CRYPT_KEY_PROV_PARAM
    dwParam As Long
    pbData As Long
    cbData As Long
    dwFlags As Long
End Type

Private Type CRYPT_KEY_PROV_INFO
    pwszContainerName As Long
    pwszProvName As Long
    dwProvType As Long
    dwFlags As Long
    cProvParam As Long
    rgProvParam As Long
    dwKeySpec As Long
End Type

Private Type CERT_KEY_CONTEXT
    cbSize As Long           ' sizeof(CERT_KEY_CONTEXT)
    hCryptProv As Long
    dwKeySpec As Long
End Type

'--- WinCrypt API Declarations
Private Declare Function CryptAcquireContext Lib "advapi32.dll" Alias "CryptAcquireContextA" ( _
    phProv As Long, _
    ByVal pszContainer As String, _
    ByVal pszProvider As String, _
    ByVal dwProvType As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptReleaseContext Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptGenKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptDeriveKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hBaseData As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptDestroyKey Lib "advapi32.dll" ( _
    ByVal hKey As Long) As Long

Private Declare Function CryptGetHashParam Lib "advapi32.dll" ( _
    ByVal hHash As Long, _
    ByVal dwParam As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptGetUserKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal dwKeySpec As Long, _
    phUserKey As Long) As Long

Private Declare Function CryptExportKey Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hExpKey As Long, _
    ByVal dwBlobType As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long) As Long

Private Declare Function CryptImportKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal pbData As String, _
    ByVal dwDataLen As Long, _
    ByVal hPubKey As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptEncrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long, _
    ByVal dwBufLen As Long) As Long

Private Declare Function CryptDecrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long) As Long

Private Declare Function CryptCreateHash Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hKey As Long, _
    ByVal dwFlags As Long, _
    phHash As Long) As Long

Private Declare Function CryptHashData Lib "advapi32.dll" ( _
    ByVal hHash As Long, _
    ByVal pbData As String, _
    ByVal dwDataLen As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptDestroyHash Lib "advapi32.dll" ( _
    ByVal hHash As Long) As Long

Private Declare Function CertOpenSystemStore Lib "Crypt32.dll" Alias "CertOpenSystemStoreA" ( _
    ByVal hProv As Long, _
    ByVal szSubsystemProtocol As String _
    ) As Long

Private Declare Function CertEnumCertificatesInStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal pPrevCertContext As Long _
    ) As Long

Private Declare Function CertEnumCertificateContextProperties Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long _
    ) As Long

Private Declare Function CertGetCertificateContextProperty Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByVal pvData As String, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextDWProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As Long, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextKCProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As CERT_KEY_CONTEXT, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextKPIProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As CRYPT_KEY_PROV_INFO, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertFreeCertificateContext Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long _
    ) As Long

Private Declare Function CertCloseStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal dwFlags As Long _
    ) As Long

Private Declare Function CertGetNameString Lib "Crypt32.dll" Alias "CertGetNameStringA" ( _
    ByVal pCertContext As Long, _
    ByVal dwType As Long, _
    ByVal dwFlags As Long, _
    ByRef pvTypePara As Long, _
    ByVal pszNameString As String, _
    ByVal cchNameString As Long _
    ) As Long

'constants from WinErr.h
Private Const ERROR_INVALID_HANDLE As Long = 6
Private Const ERROR_INVALID_PARAMETER As Long = 87
Private Const NTE_BAD_KEY As Long = &H80090003
Private Const NTE_BAD_UID As Long = &H80090001
Private Const NTE_NO_KEY As Long = &H8009000D
Private Const NTE_BAD_SIGNATURE As Long = &H80090006

' Private property buffers
Private m_sPassword As String   ' Password used to create encryption key

Private m_sInBuffer As String   ' Used as an input buffer for all
                                ' data to be encrypted or decrypted

Private m_sOutBuffer As String  ' Used as an output buffer for all
                                ' data that has been encrypted or decrypted

Private m_sExportKey As String  ' Used to publish the exported public
                                ' key that can be sent to the other
                                ' application that is to be sending
                                ' encrypted data to this class

Private m_sImportKey As String  ' Used to import the public key of
                                ' the other application to which this
                                ' application will be sending encrypted
                                ' data

Private m_sSessionKey As String ' Used to publish and import the
                                ' encrypted session key that is used
                                ' to encrypt and decrypt the data to
                                ' be passed between applications
                                
Private m_sErrorMsg As String   ' Error message string

' Private class-level variables
Private m_lHCryptProv As Long   ' Handle for the cryptographic
                                ' service provider (CSP)

Private m_lHCryptKey As Long    ' Handle for the AT_KEYEXCHANGE
                                ' (Public/Private) key for this
                                ' application

Private m_lHImportKey As Long   ' Handle for the imported public
                                ' key of the application to which
                                ' this application will be sending
                                ' data

Private m_lHSessionKey As Long  ' Session key for encrypting and
                                ' decrypting data

Private m_sHashBuffer As String ' Used as a buffer for all the hash value

Private m_iHashLength As Long   ' Used to hold the hash length


Public Property Get InBuffer() As String
    InBuffer = m_sInBuffer
End Property

Public Property Let InBuffer(vNewValue As String)
    m_sInBuffer = vNewValue
End Property

Public Property Get OutBuffer() As String
    OutBuffer = m_sOutBuffer
End Property

Public Property Get ExportKey() As String
    ExportKey = m_sExportKey
End Property

Public Property Get SessionKey() As String
    SessionKey = m_sSessionKey
End Property

Public Property Let SessionKey(vNewValue As String)
    m_sSessionKey = vNewValue
End Property

Public Property Get ImportKey() As String
    ImportKey = m_sImportKey
End Property

Public Property Let ImportKey(vNewValue As String)
    m_sImportKey = vNewValue
End Property

Public Property Get ErrorMsg() As String
    ErrorMsg = m_sErrorMsg
End Property

Private Sub Class_Initialize()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Initialize
'*
'* Parameters:  None
'*
'* Purpose: Calls the InitUser function to initialize the
'*          class. The initialization process loads the CSP
'*          and Public/Private key.
'*************************************************************
    If Not InitUser Then
        m_sErrorMsg = "Unable to initialize CryptoAPI."
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
    End If
End Sub

Private Function InitUser() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     InitUser
'*
'* Parameters:  None
'*
'* Purpose: This function loads the default CSP (specified
'*          in the defined constant MS_DEF_PROV), and the
'*          AT_KEYEXCHANGE keyset. If the keyset or key
'*          container does not exist, then this function
'*          creates them.
'*************************************************************
On Error Resume Next
    InitUser = False
    
    Dim lDataSize As Long
    
    Dim lResult As Long
    Dim sResult As String
    Dim sContainer As String
    Dim sProvider As String
    
    '--- Prepare string buffers
    sContainer = vbNullChar
    sProvider = MS_DEF_PROV & vbNullChar
    
    '--- Attempt to acquire a handle to the default key container.
    If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, 0)) Then
    
        '--- Create default key container.
        If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, CRYPT_NEWKEYSET)) Then
            m_sErrorMsg = "Error creating key container - " & CStr(Err.LastDllError)
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
            
            Exit Function
        End If
        
    End If
    
    '--- Didn't exit early, return TRUE
    InitUser = True

    Exit Function
    
End Function

Public Function ExportPublicKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     ExportPublicKey
'*
'* Parameters:  None
'*
'* Purpose: This function exports the public key from the
'*          AT_KEYEXCHANGE keyset, placing the exported key
'*          into the ExportKey property
'*************************************************************
On Error Resume Next
    ExportPublicKey = False

    Dim lDataSize As Long
    Dim lResult As Long

    '--- Null the target string, so we can get the size of the string
    '--- that will be exported
    lDataSize = 0

    '--- Call CryptExportKey with the nulled string. This will give us
    '--- the required string size that will be needed to hold the exported
    '--- key
    lResult = CryptExportKey(m_lHCryptKey, 0, PUBLICKEYBLOB, 0, vbNullString, lDataSize)

    '--- Size the target string for the size specified, filling it with
    '--- null characters
    m_sExportKey = String(lDataSize, vbNullChar)

    '--- Export the public key
    If Not CBool(CryptExportKey(m_lHCryptKey, 0, PUBLICKEYBLOB, 0, m_sExportKey, lDataSize)) Then
        m_sErrorMsg = "Error during CryptExportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ExportPublicKey = True

    Exit Function

End Function

Public Function ImportPublicKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     ImportPublicKey
'*
'* Parameters:  None
'*
'* Purpose: This will import the public key of the other
'*          application, taking it from the ImportKey property.
'*          The imported public key will be held in the
'*          m_lHImportKey class variable.
'*************************************************************
On Error Resume Next
    ImportPublicKey = False

    Dim lDataSize As Long

    '--- Get the size of the key being imported
    lDataSize = Len(m_sImportKey)

    '--- Import the key, storing it in the m_lHImportKey class variable
    If Not CBool(CryptImportKey(m_lHCryptProv, m_sImportKey, lDataSize, 0, 0, m_lHImportKey)) Then
        m_sErrorMsg = "Error during CryptImportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ImportPublicKey = True

    Exit Function

End Function

Private Sub Class_Terminate()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Terminate
'*
'* Parameters:  None
'*
'* Purpose: This is executed when this class is destroyed. This
'*          subroutine examines each of the class-level handles
'*          for the keys and keycontext, and destroys or
'*          releases them as is appropriate to clean up after
'*          oneself.
'*************************************************************
    Dim lResult As Long
    
    '--- Do we have an imported key? If so, destroy it.
    If (m_lHImportKey <> 0) Then lResult = CryptDestroyKey(m_lHImportKey)

    '--- Do we have a Public/Private keypair? If so, destroy it
    If (m_lHCryptKey <> 0) Then lResult = CryptDestroyKey(m_lHCryptKey)

    '--- Do we have an open key context? If so, release it.
    If (m_lHCryptProv <> 0) Then lResult = CryptReleaseContext(m_lHCryptProv, 0)
End Sub


Public Function GeneratePasswordKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     GeneratePasswordKey
'*
'* Parameters:  None
'*
'* Purpose: This will generate a session key from a hash of
'*          the password entered by the user. The session key
'*          will be placed into a private class variable.
'*************************************************************
On Error Resume Next
    Dim lHash As Long
    Dim lResult As Long
    
    GeneratePasswordKey = False
    
    '--- Create an empty hash object.
    If Not CBool(CryptCreateHash(m_lHCryptProv, CALG_MD5, 0, 0, lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptCreateHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Hash the password string.
    If Not CBool(CryptHashData(lHash, m_sPassword, Len(m_sPassword), 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptHashData!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Create a derived block cipher session key.
    If Not CBool(CryptDeriveKey(m_lHCryptProv, CALG_RC2, lHash, 0, m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDeriveKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Destroy the hash object
    If Not CBool(CryptDestroyHash(lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    GeneratePasswordKey = True
End Function

Public Property Get Password() As String
    Password = m_sPassword
End Property

Public Property Let Password(ByVal sNewValue As String)
    m_sPassword = sNewValue
End Property

Public Function EncryptMessageData()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     EncryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This function will take the data in the InBuffer property
'*          and encrypt it using the session key. The resulting
'*          encrypted data is placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    EncryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Determine the size of the buffer needed for encrypting the data
    '--- in the InBuffer property
    lCryptLength = Len(m_sInBuffer)
    lResult = CryptEncrypt(m_lHSessionKey, 0, 1, 0, vbNullString, lCryptLength, lCryptBufLen)
    
    '--- Prepare a string buffer for the CryptEncrypt function
    lCryptBufLen = lCryptLength * 2
    lCryptLength = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    
    '--- Copy in the contents of the InBuffer property
    LSet sCryptBuffer = m_sInBuffer

    '--- Encrypt data
    If Not CBool(CryptEncrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptLength, lCryptBufLen)) Then
        
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptEncrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If

    '--- Copy the encrypted data to the OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptLength)

    EncryptMessageData = True
End Function

Public Function DestroySessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DestroySessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will destroy the session key.
'*************************************************************
    Dim lResult As Long
    
    DestroySessionKey = False
    
    '--- Destroy the session key
    If Not CBool(CryptDestroyKey(m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    
    DestroySessionKey = True
End Function

Public Function DecryptMessageData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DecryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This will use the session key to decrypt the data in
'*          the InBuffer property. The decrypted data is then
'*          placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    DecryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Prepare sCryptBuffer for CryptDecrypt
    lCryptBufLen = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    LSet sCryptBuffer = m_sInBuffer
    
    '--- Decrypt data
    If Not CBool(CryptDecrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptBufLen)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDecrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    '--- Apply decrypted string from sCryptBuffer to private buffer for OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptBufLen)
    
    '--- Didn't exit early, return TRUE
    DecryptMessageData = True
End Function

Public Function HashFile() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     HashFile
'*
'* Parameters:  None
'*
'* Purpose: This function creates a hash value for the contents
'*          of the input buffer. The resulting hash value will
'*          be placed in the m_sHashBuffer variable for later
'*          use.
'*************************************************************
On Error Resume Next
    Dim lHash As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptBufLen As Long

    HashFile = False
    
    '--- Create an empty hash object.
    If Not CBool(CryptCreateHash(m_lHCryptProv, CALG_MD5, 0, 0, lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptCreateHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Hash the password string.
    If Not CBool(CryptHashData(lHash, m_sInBuffer, Len(m_sInBuffer), 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptHashData!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Extract the hash value size.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHSIZE, sCryptBuffer, lCryptBufLen, 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGetHashParam!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Extract the hash value buffer size.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHVAL, vbNullString, lCryptBufLen, 0)) Then
        '--- Ignore, we expect this to give an error. But it does give us the size needed
        '--- for the hash buffer
    End If
    
    '--- Prepare sCryptBuffer for CryptDecrypt
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    
    '--- Extract the hash value.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHVAL, sCryptBuffer, lCryptBufLen, 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGetHashParam!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Apply hash string from sCryptBuffer to private buffer for HashBuffer variables
    m_sHashBuffer = Mid$(sCryptBuffer, 1, lCryptBufLen)
    m_iHashLength = lCryptBufLen
    
    '--- Didn't exit early, return TRUE
    HashFile = True
End Function

Public Function EncryptFileData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     EncryptFileData
'*
'* Parameters:  None
'*
'* Purpose: This function places the hash of the original file
'*          at the front of the file (preceded by the hash
'*          length) and the calls EncryptMessageData to encrypt
'*          the whole thing.
'*************************************************************
    EncryptFileData = False
    
    Dim strInputData As String
    
    '--- Concatenate the length of the hash, the hash value, and the original
    '--- file contents
    strInputData = Trim(Str(m_iHashLength)) + m_sHashBuffer + m_sInBuffer
    
    '--- Copy the new string to the input buffer
    m_sInBuffer = strInputData
    
    '--- Call EncryptMessageData and return it's return value
    EncryptFileData = EncryptMessageData
End Function

Public Function DecryptFileData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DecryptFileData
'*
'* Parameters:  None
'*
'* Purpose: This function calls DecryptMessageData to decrypt
'*          the file contents. The original hash value is then
'*          stripped off the front of the decrypted data, and
'*          a new hash of the decrypted data is generated. If
'*          the two hash values match, the file has not been
'*          corrupted. If the two values don't match, the file
'*          has been corrupted.
'*************************************************************
    DecryptFileData = False
    
    Dim strInputData As String
    Dim strHash As String
    Dim strHashLength As String
    Dim lHashLength As Long
    
    '--- Decrypt the file data
    If Not DecryptMessageData Then Exit Function
    
    '--- Extract the original hash length
    strHashLength = Left(m_sOutBuffer, 2)
    lHashLength = CLng(strHashLength)
    
    '--- Extract the original hash value
    strHash = Mid(m_sOutBuffer, 3, lHashLength)
    
    '--- Extract the original file contents
    strInputData = Mid(m_sOutBuffer, (3 + lHashLength))
    
    '--- Copy the original file contents to the input buffer
    m_sInBuffer = strInputData
    
    '--- Hash the original file contents
    If Not HashFile Then Exit Function
        
    '--- Compare the original hash value to the new hash value
    If (strHash = m_sHashBuffer) Then
        '--- If they match, the file is unaltered, copy the original
        '--- file contents to the output buffer
        m_sOutBuffer = strInputData
    Else
        '--- The hash values don't match, the file has been corrupted
        MsgBox "File has been corrupted since original encryption."
        m_sOutBuffer = ""
        Exit Function
    End If
    
    '--- Didn't exit early, return TRUE
    DecryptFileData = True
End Function


Public Function GetExchangeKeypair() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     GetExchangeKeypair
'*
'* Parameters:  None
'*
'* Purpose: This function loads the AT_KEYEXCHANGE keyset.
'*          If the keyset or key container does not exist,
'*          then this function creates them.
'*************************************************************
    Dim lResult As Long
    Dim sResult As String

    GetExchangeKeypair = False
    
    '--- Attempt to get handle to exchange key
    If Not CBool(CryptGetUserKey(m_lHCryptProv, AT_KEYEXCHANGE, m_lHCryptKey)) Then

        '--- Determine what the error was
        lResult = Err.LastDllError

        '--- If the problem is that there is no existing key, generate one
        If ((lResult = NTE_NO_KEY) Or (lResult = 0)) Then
            If Not CBool(CryptGenKey(m_lHCryptProv, AT_KEYEXCHANGE, 0, m_lHCryptKey)) Then
                m_sErrorMsg = "Error during CryptGenKey - " & CStr(Err.LastDllError)
                MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

                Exit Function
            End If
        Else

            '--- If there was another cause for the error, inform the user
            '--- what the problem is
            Select Case lResult

                Case ERROR_INVALID_HANDLE
                    sResult = "Invalid handle"

                Case ERROR_INVALID_PARAMETER
                    sResult = "Invalid parameter"

                Case NTE_BAD_KEY
                    sResult = "Bad key"

                Case NTE_BAD_UID
                    sResult = "Bad handle"
            End Select
            m_sErrorMsg = "Error during CryptGetUserKey - " + sResult
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

            Exit Function
        End If
    End If

    '--- Didn't exit early, return TRUE
    GetExchangeKeypair = True
End Function

Public Function CreateAndExportSessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 07, 1999
'*
'* Syntax:     CreateAndExportSessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will generate a session key and export it,
'*          encrypting the exported key using the public key
'*          of the recipient. The exported session key will
'*          be placed in the SessionKey property.
'*************************************************************
On Error Resume Next
    CreateAndExportSessionKey = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Create a random block cipher session key.
    If Not CBool(CryptGenKey(m_lHCryptProv, CALG_RC2, CRYPT_EXPORTABLE, m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGenKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Determine the size of the key blob and allocate memory.
    lResult = CryptExportKey(m_lHSessionKey, m_lHImportKey, SIMPLEBLOB, 0, vbNullString, lDataSize)
    m_sSessionKey = String(lDataSize, vbNullChar)

    '--- Export the session key, using the imported public key to encrypt it
    If Not CBool(CryptExportKey(m_lHSessionKey, m_lHImportKey, SIMPLEBLOB, 0, m_sSessionKey, lDataSize)) Then
        m_sErrorMsg = "Error during CryptExportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    CreateAndExportSessionKey = True
End Function

Public Function ImportSessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 07, 1999
'*
'* Syntax:     ImportSessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will import the session key from the SessionKey
'*          property.
'*************************************************************
On Error Resume Next
    ImportSessionKey = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Determine the size of the session key to be imported
    lDataSize = Len(m_sSessionKey)

    '--- Import the session key. As it should have been encrypted using
    '--- our public key, the CAPI should know to use our private key to
    '--- decrypt the session key during the import
    If Not CBool(CryptImportKey(m_lHCryptProv, m_sSessionKey, lDataSize, 0, 0, m_lHSessionKey)) Then
        m_sErrorMsg = "Error during CryptImportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ImportSessionKey = True
End Function

Private Function GetCSPFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetCSPFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the CSP from the CRYPT_KEY_PROV_INFO
'*          type structure.
'*************************************************************
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    
    '--- Initialize the starting position
    iCurPos = 29
    '--- Trim off the CRYPT_KEY_PROV_INFO structure
    strChar = Mid(sbOrg, iCurPos)
    '--- Convert the string
    strRtn = StrConv(strChar, vbFromUnicode)
    '--- Find the end of the first string
    iCurPos = InStr(1, strRtn, vbNullChar, vbBinaryCompare)
    '--- Trim off the first string
    If iCurPos > 0 Then strRtn = Mid(strRtn, (iCurPos + 3))
    '--- Find the end of the second string
    iCurPos = InStr(1, strRtn, vbNullChar, vbBinaryCompare)
    '--- Trim the string one character shy of the NULL character
    If iCurPos > 0 Then strRtn = Left(strRtn, (iCurPos - 1))
    '--- Return the CSP name
    GetCSPFromKPI = strRtn
End Function

Private Function GetContainerNameFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetContainerNameFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the container name from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    
    '--- Extract the container name, starting at the
    '--- appropriate offset
    strRtn = ConvertBytesToString(Mid(sbOrg, 29))
    '--- Return the container name
    GetContainerNameFromKPI = strRtn
End Function

Private Function ConvertBytesToString(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     ConvertBytesToString(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will convert a byte array value to a string
'*          value by skipping every other character (byte) in
'*          the original string.
'*************************************************************
    Dim strNew As String
    Dim strCur As String
    Dim iCurPos As Integer
    
    '--- Convert the string from Unicode
    strNew = StrConv(sbOrg, vbFromUnicode)
    '--- Find any NULL characters in the string
    iCurPos = InStr(1, strNew, vbNullChar, vbBinaryCompare)
    '--- If any NULL characters were found, trim the string
    '--- one character shy of the NULL character
    If iCurPos > 0 Then strNew = Left(strNew, (iCurPos - 1))
    '--- Return the extracted string
    ConvertBytesToString = strNew
End Function

Public Function ListCerts(pszStoreName As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     ListCerts(pszStoreName)
'*
'* Parameters:  pszStoreName As String
'*
'* Purpose: This will loop through all of the certificates in
'*          the specified store, listing the available
'*          properties of each in turn. It builds this
'*          information as a large text string that is returned
'*          to the calling function to be displayed to the user.
'*************************************************************

    On Error GoTo ListCertsErr
    
    Dim hCertStore As Long
    Dim pszNameString As String * 256
    Dim pvData As String
    Dim dwData As Long
    Dim kcData As CERT_KEY_CONTEXT
    Dim kpiData As CRYPT_KEY_PROV_INFO
    Dim cbData As Long
    Dim dwPropId As Long
    Dim strMsg As String
    Dim lpCertContext As Long
    Dim szStoreName As String
    Dim szRtnMsg As String
    Dim iPos As Integer
    Dim iVerToUse As Integer
    
    szRtnMsg = ""
    dwPropId = 0

    '--- Open a system certificate store.
    hCertStore = CertOpenSystemStore(0, pszStoreName)
    
    '--- Did we open a store?
    If (hCertStore <> 0) Then
        szRtnMsg = "The " + pszStoreName + " store has been opened." + vbCrLf
    Else
        m_sErrorMsg = "Error during CertOpenSystemStore - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If

    '--- Use CertEnumCertificatesInStore to get the certificates
    '--- from the open store.
    lpCertContext = CertEnumCertificatesInStore( _
        hCertStore, _
        0)

    '--- Loop until all certificates have been retrieved
    While (lpCertContext <> 0)
        
        '--- A certificate was retrieved. Get the name from the certificate.
        If (CertGetNameString( _
                lpCertContext, _
                CERT_NAME_SIMPLE_DISPLAY_TYPE, _
                0, _
                0, _
                pszNameString, _
                128)) Then
                
            '--- Find the end of the name
            iPos = InStr(1, pszNameString, vbNullChar, vbBinaryCompare)
            '--- Strip off the remaining NULL characters
            If iPos > 0 Then pszNameString = Left(pszNameString, (iPos - 1))
            '--- Tell the user the name of the certificate
            szRtnMsg = szRtnMsg + vbCrLf + "Certificate for " + Trim$(pszNameString) + vbCrLf
        Else
            m_sErrorMsg = "Error during CertGetNameString - " & CStr(Err.LastDllError) & Error$
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        End If

        '--- Loop to find all of the property identifiers for the specified
        '--- certificate. The loop continues until
        '--- CertEnumCertificateContextProperties returns zero.

        dwPropId = CertEnumCertificateContextProperties( _
                lpCertContext, _
                dwPropId)
        While (dwPropId <> 0)
        
            '--- When the loop is executed, a property identifier has been found.
            '--- Print the property number.
            strMsg = "Property # " + Str(dwPropId) + " found -> "

            '--- Initialize the property data type indicator
            iVerToUse = 1
            
            '---  Indicate the kind of property found.
            Select Case dwPropId
                Case CERT_FRIENDLY_NAME_PROP_ID
                    strMsg = strMsg + "Friendly name: "
                    iVerToUse = 5
                Case CERT_SIGNATURE_HASH_PROP_ID
                    strMsg = strMsg + "Signature hash identifier "
                Case CERT_KEY_PROV_HANDLE_PROP_ID
                    strMsg = strMsg + "KEY PROVE HANDLE "
                    iVerToUse = 2
                Case CERT_KEY_PROV_INFO_PROP_ID
                    strMsg = strMsg + "KEY PROV INFO PROP ID "
                    iVerToUse = 4
                Case CERT_SHA1_HASH_PROP_ID
                    strMsg = strMsg + "SHA1 HASH identifier "
                Case CERT_MD5_HASH_PROP_ID
                    strMsg = strMsg + "md5 hash identifier "
                Case CERT_KEY_CONTEXT_PROP_ID
                    strMsg = strMsg + "KEY CONTEXT PROP identifier "
                Case CERT_KEY_SPEC_PROP_ID
                    strMsg = strMsg + "KEY SPEC PROP identifier "
                    iVerToUse = 2
                Case CERT_ENHKEY_USAGE_PROP_ID
                    strMsg = strMsg + "ENHKEY USAGE PROP identifier "
                Case CERT_NEXT_UPDATE_LOCATION_PROP_ID
                    strMsg = strMsg + "NEXT UPDATE LOCATION PROP identifier "
                Case CERT_PVK_FILE_PROP_ID
                    strMsg = strMsg + "PVK FILE PROP identifier "
                Case CERT_DESCRIPTION_PROP_ID
                    strMsg = strMsg + "DESCRIPTION PROP identifier "
                    iVerToUse = 5
                Case CERT_ACCESS_STATE_PROP_ID
                    strMsg = strMsg + "ACCESS STATE PROP identifier "
                    iVerToUse = 2
                Case CERT_SMART_CARD_DATA_PROP_ID
                    strMsg = strMsg + "SMART_CARD DATA PROP identifier "
                Case CERT_EFS_PROP_ID
                    strMsg = strMsg + "EFS PROP identifier "
                Case CERT_FORTEZZA_DATA_PROP_ID
                    strMsg = strMsg + "FORTEZZA DATA PROP identifier "
                Case CERT_ARCHIVED_PROP_ID
                    strMsg = strMsg + "ARCHIVED PROP identifier "
                    iVerToUse = 2
                Case CERT_KEY_IDENTIFIER_PROP_ID
                    strMsg = strMsg + "KEY IDENTIFIER PROP identifier "
                Case CERT_AUTO_ENROLL_PROP_ID
                    strMsg = strMsg + "AUTO ENROLL identifier. "
            End Select

            '--- Determine how to retrieve the property value based on the
            '--- property type
            Select Case iVerToUse
                Case 2
                    '--- Set the value size
                    cbData = 4
                    '--- Retrieve the property value
                    If (Not CBool(CertGetCertificateContextDWProperty( _
                            lpCertContext, _
                            dwPropId, _
                            dwData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextDWProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    strMsg = strMsg + vbCrLf + "The Property Content is " + Str(dwData)
                    
                Case 3
                    '--- Set the value size
                    cbData = 12
                    '--- Retrieve the property value
                    If (Not CBool(CertGetCertificateContextKCProperty( _
                            lpCertContext, _
                            dwPropId, _
                            kcData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextKCProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    '--- Show the results.
                    strMsg = strMsg + vbCrLf + "The Property Content is " + Str(kcData.dwKeySpec)
                
                Case 4
                    '--- Set the value size
                    cbData = 28
                    '--- Retrieve the property value
                    If (CBool(CertGetCertificateContextKPIProperty( _
                            lpCertContext, _
                            dwPropId, _
                            kpiData, _
                            cbData))) Then
                    End If
                    
                    '--- The call didn't succeed. Use the size to allocate memory
                    '--- for the property.
                    pvData = String(cbData, vbNullChar)
                    
                    '--- Allocation succeeded. Retrieve the property data.
                    If (Not CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            pvData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    If cbData = 28 Then
                        '--- We retrieved the type structure
                        strMsg = strMsg + vbCrLf + "The Property Content is " + Str(kpiData.dwProvType)
                    Else
                        '--- We received a byte array
                        strMsg = strMsg + vbCrLf + "The Container Name is " + GetContainerNameFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The CSP is " + GetCSPFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The Provider Type is " + GetProvTypeFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The Key Type is " + GetKeySpecFromKPI(pvData)
                    End If
                Case Else
                    '--- Retrieve information on the property by first getting the size
                    '--- of the property size.
                    cbData = 0
                    If CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            vbNullString, _
                            cbData)) Then
                    End If
                    
                    '--- The call succeeded. Use the size to allocate memory
                    '--- for the property.
                    pvData = String(cbData, vbNullChar)
                    
                    '--- Allocation succeeded. Retrieve the property data.
                    If (Not CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            pvData, _
                            cbData))) Then
                        m_sErrorMsg = "Error during CertGetCertificateContextProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    If iVerToUse = 5 Then
                        '--- The value is a string, extract it from the Byte array
                        strMsg = strMsg + vbCrLf + "The Property Content is " + ConvertBytesToString(pvData)
                    Else
                        '--- A byte array, display the ASCII value of the first character
                        strMsg = strMsg + vbCrLf + "The Property Content is " + Str(Asc(pvData))
                    End If
            End Select
            
            '--- Add the value to the return string
            szRtnMsg = szRtnMsg + strMsg + vbCrLf
            
            '--- Get the next property
            dwPropId = CertEnumCertificateContextProperties( _
                lpCertContext, _
                dwPropId)
        Wend
    
        '--- Get the next certificate
        lpCertContext = CertEnumCertificatesInStore( _
            hCertStore, _
            lpCertContext)
    Wend
    

    '--- Clean up.
    '--- If we have a current certificate, then release it
    If lpCertContext <> 0 Then
        CertFreeCertificateContext lpCertContext
    End If
    '--- Close the Certificate store
    CertCloseStore hCertStore, 0
    szRtnMsg = szRtnMsg + vbCrLf + "The function completed successfully."
    ListCerts = szRtnMsg
    
    Exit Function

ListCertsErr:
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function


Private Function GetProvTypeFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetProvTypeFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the provider type from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    Dim lProvType As Long
    
    '--- Extract the provider type, starting at the
    '--- appropriate offset
    lProvType = Asc(Mid(sbOrg, 12, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 11, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 10, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 9, 1))
    '--- Determine the provider type
    Select Case lProvType
        Case PROV_RSA_FULL
            strRtn = "PROV_RSA_FULL"
        Case PROV_RSA_SIG
            strRtn = "PROV_RSA_SIG"
        Case PROV_DSS
            strRtn = "PROV_DSS"
        Case PROV_FORTEZZA
            strRtn = "PROV_FORTEZZA"
        Case PROV_MS_EXCHANGE
            strRtn = "PROV_MS_EXCHANGE"
        Case PROV_SSL
            strRtn = "PROV_SSL"
        Case PROV_RSA_SCHANNEL
            strRtn = "PROV_RSA_SCHANNEL"
        Case PROV_DSS_DH
            strRtn = "PROV_DSS_DH"
    End Select
    '--- Return the provider type
    GetProvTypeFromKPI = strRtn
End Function

Private Function GetKeySpecFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetKeySpecFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the key spec from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    Dim lKeySpec As Long
    
    '--- Extract the key spec, starting at the
    '--- appropriate offset
    lKeySpec = Asc(Mid(sbOrg, 28, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 27, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 26, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 25, 1))
    '--- Determine the key spec
    Select Case lKeySpec
        Case AT_KEYEXCHANGE
            strRtn = "AT_KEYEXCHANGE"
        Case AT_SIGNATURE
            strRtn = "AT_SIGNATURE"
    End Select
    '--- Return the key spec
    GetKeySpecFromKPI = strRtn
End Function

