VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCrypto"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Purpose: This class encapsulates a small amount of the
'*          Microsoft CryptoAPI functionality. As of the
'*          initial creation, it provides functionality to
'*          exchange public keys and pass messages using the
'*          AT_KEYEXCHANGE keys only. A session key is
'*          generated for each message encrypted, so the
'*          initial version of this class is not suited for
'*          encrypting large amounts of data on a continuous
'*          basis. To support that capability, this class
'*          would need to be enhanced to include a class-level
'*          session key that was maintained and used for all
'*          encrypting and decrypting.
'*
'* Revisions
'*   Name          Date     Description
'*   ---------------------------------------------------------
'*************************************************************
'+---------------------------------------------------------------------------
'
'  Microsoft Windows
'  Copyright 1992 - 1998 Microsoft Corporation.
'
'  File:       wincrypt.h
'
'  Contents:   Cryptographic API Prototypes and Definitions
'
'----------------------------------------------------------------------------


'
' Algorithm IDs and Flags
'

' Algorithm classes
Private Const ALG_CLASS_ANY = 0
Private Const ALG_CLASS_SIGNATURE = 8192
Private Const ALG_CLASS_MSG_ENCRYPT = 16384
Private Const ALG_CLASS_DATA_ENCRYPT = 24576
Private Const ALG_CLASS_HASH = 32768
Private Const ALG_CLASS_KEY_EXCHANGE = 40960

' Algorithm types
Private Const ALG_TYPE_ANY = 0
Private Const ALG_TYPE_DSS = 512
Private Const ALG_TYPE_RSA = 1024
Private Const ALG_TYPE_BLOCK = 1536
Private Const ALG_TYPE_STREAM = 2048
Private Const ALG_TYPE_DH = 2560
Private Const ALG_TYPE_SECURECHANNEL = 3072

' RC2 sub-ids
Private Const ALG_SID_RC2 = 2

' Stream cipher sub-ids
Private Const ALG_SID_RC4 = 1
Private Const ALG_SID_SEAL = 2

' Diffie-Hellman sub-ids
Private Const ALG_SID_DH_SANDF = 1
Private Const ALG_SID_DH_EPHEM = 2
Private Const ALG_SID_AGREED_KEY_ANY = 3
Private Const ALG_SID_KEA = 4

' Hash sub ids
Private Const ALG_SID_MD2 = 1
Private Const ALG_SID_MD4 = 2
Private Const ALG_SID_MD5 = 3
Private Const ALG_SID_SHA = 4
Private Const ALG_SID_SHA1 = 4
Private Const ALG_SID_MAC = 5
Private Const ALG_SID_RIPEMD = 6
Private Const ALG_SID_RIPEMD160 = 7
Private Const ALG_SID_SSL3SHAMD5 = 8
Private Const ALG_SID_HMAC = 9

' algorithm identifier definitions
Private Const CALG_MD5 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MD5)
Private Const CALG_RC2 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_RC2)

' dwFlags definitions for CryptAcquireContext
Private Const CRYPT_VERIFYCONTEXT = &HF0000000
Private Const CRYPT_NEWKEYSET = &H8
Private Const CRYPT_DELETEKEYSET = &H10
Private Const CRYPT_MACHINE_KEYSET = &H20

' dwFlag definitions for CryptGenKey
Private Const CRYPT_EXPORTABLE = &H1
Private Const CRYPT_USER_PROTECTED = &H2
Private Const CRYPT_CREATE_SALT = &H4
Private Const CRYPT_UPDATE_KEY = &H8
Private Const CRYPT_NO_SALT = &H10
Private Const CRYPT_PREGEN = &H40
Private Const CRYPT_RECIPIENT = &H10
Private Const CRYPT_INITIATOR = &H40
Private Const CRYPT_ONLINE = &H80
Private Const CRYPT_SF = &H100
Private Const CRYPT_CREATE_IV = &H200
Private Const CRYPT_KEK = &H400
Private Const CRYPT_DATA_KEY = &H800

' dwFlags definitions for CryptDeriveKey
Private Const CRYPT_SERVER = &H400

Private Const KEY_LENGTH_MASK = &HFFFF0000

' dwFlag definitions for CryptSetProviderEx and CryptGetDefaultProvider
Private Const CRYPT_MACHINE_DEFAULT = &H1
Private Const CRYPT_USER_DEFAULT = &H2
Private Const CRYPT_DELETE_DEFAULT = &H4

'
' CryptSetProvParam
'
Private Const PROV_RSA_FULL = 1
Private Const PROV_RSA_SIG = 2
Private Const PROV_DSS = 3
Private Const PROV_FORTEZZA = 4
Private Const PROV_MS_EXCHANGE = 5
Private Const PROV_SSL = 6
Private Const PROV_RSA_SCHANNEL = 12
Private Const PROV_DSS_DH = 13
Private Const PROV_EC_ECDSA_SIG = 14
Private Const PROV_EC_ECNRA_SIG = 15
Private Const PROV_EC_ECDSA_FULL = 16
Private Const PROV_EC_ECNRA_FULL = 17
Private Const PROV_SPYRUS_LYNKS = 20

'
' Provider friendly names
'
Private Const MS_DEF_PROV = "Microsoft Base Cryptographic Provider v1.0"
Private Const MS_ENHANCED_PROV = "Microsoft Enhanced Cryptographic Provider v1.0"
Private Const MS_DEF_RSA_SIG_PROV = "Microsoft RSA Signature Cryptographic Provider"
Private Const MS_DEF_RSA_SCHANNEL_PROV = "Microsoft Base RSA SChannel Cryptographic Provider"
Private Const MS_ENHANCED_RSA_SCHANNEL_PROV = "Microsoft Enhanced RSA SChannel Cryptographic Provider"
Private Const MS_DEF_DSS_PROV = "Microsoft Base DSS Cryptographic Provider"
Private Const MS_DEF_DSS_DH_PROV = "Microsoft Base DSS and Diffie-Hellman Cryptographic Provider"


'--- WinCrypt API Declarations
Private Declare Function CryptAcquireContext Lib "advapi32.dll" Alias "CryptAcquireContextA" ( _
    phProv As Long, _
    ByVal pszContainer As String, _
    ByVal pszProvider As String, _
    ByVal dwProvType As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptReleaseContext Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptDeriveKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hBaseData As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptDestroyKey Lib "advapi32.dll" ( _
    ByVal hKey As Long) As Long

Private Declare Function CryptEncrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long, _
    ByVal dwBufLen As Long) As Long

Private Declare Function CryptDecrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long) As Long

Private Declare Function CryptCreateHash Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hKey As Long, _
    ByVal dwFlags As Long, _
    phHash As Long) As Long

Private Declare Function CryptHashData Lib "advapi32.dll" ( _
    ByVal hHash As Long, _
    ByVal pbData As String, _
    ByVal dwDataLen As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptDestroyHash Lib "advapi32.dll" ( _
    ByVal hHash As Long) As Long

'constants from WinErr.h
Private Const ERROR_INVALID_HANDLE As Long = 6
Private Const ERROR_INVALID_PARAMETER As Long = 87
Private Const NTE_BAD_KEY As Long = &H80090003
Private Const NTE_BAD_UID As Long = &H80090001
Private Const NTE_NO_KEY As Long = &H8009000D
Private Const NTE_BAD_SIGNATURE As Long = &H80090006

' Private property buffers
Private m_sPassword As String   ' Password used to create encryption key

Private m_sInBuffer As String   ' Used as an input buffer for all
                                ' data to be encrypted or decrypted

Private m_sOutBuffer As String  ' Used as an output buffer for all
                                ' data that has been encrypted or decrypted

Private m_sErrorMsg As String   ' Error message string

' Private class-level variables
Private m_lHCryptProv As Long   ' Handle for the cryptographic
                                ' service provider (CSP)

Private m_lHSessionKey As Long  ' Session key for encrypting and
                                ' decrypting data

Public Property Get InBuffer() As String
    InBuffer = m_sInBuffer
End Property

Public Property Let InBuffer(vNewValue As String)
    m_sInBuffer = vNewValue
End Property

Public Property Get OutBuffer() As String
    OutBuffer = m_sOutBuffer
End Property

Public Property Get ErrorMsg() As String
    ErrorMsg = m_sErrorMsg
End Property

Private Sub Class_Initialize()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Initialize
'*
'* Parameters:  None
'*
'* Purpose: Calls the InitUser function to initialize the
'*          class. The initialization process loads the CSP
'*          and Public/Private key.
'*************************************************************
    If Not InitUser Then
        m_sErrorMsg = "Unable to initialize CryptoAPI."
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
    End If
End Sub

Private Function InitUser() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     InitUser
'*
'* Parameters:  None
'*
'* Purpose: This function loads the default CSP (specified
'*          in the defined constant MS_DEF_PROV), and the
'*          AT_KEYEXCHANGE keyset. If the keyset or key
'*          container does not exist, then this function
'*          creates them.
'*************************************************************
On Error Resume Next
    InitUser = False
    
    Dim lDataSize As Long
    
    Dim lResult As Long
    Dim sResult As String
    Dim sContainer As String
    Dim sProvider As String
    
    '--- Prepare string buffers
    sContainer = vbNullChar
    sProvider = MS_DEF_PROV & vbNullChar
    
    '--- Attempt to acquire a handle to the default key container.
    If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, 0)) Then
    
        '--- Create default key container.
        If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, CRYPT_NEWKEYSET)) Then
            m_sErrorMsg = "Error creating key container - " & CStr(Err.LastDllError)
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
            
            Exit Function
        End If
        
    End If
    
    '--- Didn't exit early, return TRUE
    InitUser = True

    Exit Function
    
End Function

Private Sub Class_Terminate()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Terminate
'*
'* Parameters:  None
'*
'* Purpose: This is executed when this class is destroyed. This
'*          subroutine examines each of the class-level handles
'*          for the keys and keycontext, and destroys or
'*          releases them as is appropriate to clean up after
'*          oneself.
'*************************************************************
    Dim lResult As Long
    
    '--- Do we have an open key context? If so, release it.
    If (m_lHCryptProv <> 0) Then lResult = CryptReleaseContext(m_lHCryptProv, 0)
End Sub

Public Function GeneratePasswordKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     GeneratePasswordKey
'*
'* Parameters:  None
'*
'* Purpose: This will generate a session key from a hash of
'*          the password entered by the user. The session key
'*          will be placed into a private class variable.
'*************************************************************
On Error Resume Next
    Dim lHash As Long
    Dim lResult As Long
    
    GeneratePasswordKey = False
    
    '--- Create an empty hash object.
    If Not CBool(CryptCreateHash(m_lHCryptProv, CALG_MD5, 0, 0, lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptCreateHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Hash the password string.
    If Not CBool(CryptHashData(lHash, m_sPassword, Len(m_sPassword), 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptHashData!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Create a derived block cipher session key.
    If Not CBool(CryptDeriveKey(m_lHCryptProv, CALG_RC2, lHash, 0, m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDeriveKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Destroy the hash object
    If Not CBool(CryptDestroyHash(lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    GeneratePasswordKey = True
End Function

Public Property Get Password() As String
    Password = m_sPassword
End Property

Public Property Let Password(ByVal sNewValue As String)
    m_sPassword = sNewValue
End Property

Public Function EncryptMessageData()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     EncryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This function will take the data in the InBuffer property
'*          and encrypt it using the session key. The resulting
'*          encrypted data is placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    EncryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Determine the size of the buffer needed for encrypting the data
    '--- in the InBuffer property
    lCryptLength = Len(m_sInBuffer)
    lResult = CryptEncrypt(m_lHSessionKey, 0, 1, 0, vbNullString, lCryptLength, lCryptBufLen)
    
    '--- Prepare a string buffer for the CryptEncrypt function
    lCryptBufLen = lCryptLength * 2
    lCryptLength = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    
    '--- Copy in the contents of the InBuffer property
    LSet sCryptBuffer = m_sInBuffer

    '--- Encrypt data
    If Not CBool(CryptEncrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptLength, lCryptBufLen)) Then
        
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptEncrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If

    '--- Copy the encrypted data to the OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptLength)

    EncryptMessageData = True
End Function

Public Function DestroySessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DestroySessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will destroy the session key.
'*************************************************************
    Dim lResult As Long
    
    DestroySessionKey = False
    
    '--- Destroy the session key
    If Not CBool(CryptDestroyKey(m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    
    DestroySessionKey = True
End Function

Public Function DecryptMessageData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DecryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This will use the session key to decrypt the data in
'*          the InBuffer property. The decrypted data is then
'*          placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    DecryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Prepare sCryptBuffer for CryptDecrypt
    lCryptBufLen = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    LSet sCryptBuffer = m_sInBuffer
    
    '--- Decrypt data
    If Not CBool(CryptDecrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptBufLen)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDecrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    '--- Apply decrypted string from sCryptBuffer to private buffer for OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptBufLen)
    
    '--- Didn't exit early, return TRUE
    DecryptMessageData = True
End Function
