VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCrypto"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Purpose: This class encapsulates a small amount of the
'*          Microsoft CryptoAPI functionality. As of the
'*          initial creation, it provides functionality to
'*          exchange public keys and pass messages using the
'*          AT_KEYEXCHANGE keys only. A session key is
'*          generated for each message encrypted, so the
'*          initial version of this class is not suited for
'*          encrypting large amounts of data on a continuous
'*          basis. To support that capability, this class
'*          would need to be enhanced to include a class-level
'*          session key that was maintained and used for all
'*          encrypting and decrypting.
'*
'* Revisions
'*   Name          Date     Description
'*   ---------------------------------------------------------
'*************************************************************
'+---------------------------------------------------------------------------
'
'  Microsoft Windows
'  Copyright 1992 - 1998 Microsoft Corporation.
'
'  File:       wincrypt.h
'
'  Contents:   Cryptographic API Prototypes and Definitions
'
'----------------------------------------------------------------------------


'
' Algorithm IDs and Flags
'

' Algorithm classes
Private Const ALG_CLASS_ANY = 0
Private Const ALG_CLASS_SIGNATURE = 8192
Private Const ALG_CLASS_MSG_ENCRYPT = 16384
Private Const ALG_CLASS_DATA_ENCRYPT = 24576
Private Const ALG_CLASS_HASH = 32768
Private Const ALG_CLASS_KEY_EXCHANGE = 40960

' Algorithm types
Private Const ALG_TYPE_ANY = 0
Private Const ALG_TYPE_DSS = 512
Private Const ALG_TYPE_RSA = 1024
Private Const ALG_TYPE_BLOCK = 1536
Private Const ALG_TYPE_STREAM = 2048
Private Const ALG_TYPE_DH = 2560
Private Const ALG_TYPE_SECURECHANNEL = 3072

'' Generic sub-ids
'Private Const ALG_SID_ANY = 0
'
'' Some RSA sub-ids
'Private Const ALG_SID_RSA_ANY = 0
'Private Const ALG_SID_RSA_PKCS = 1
'Private Const ALG_SID_RSA_MSATWORK = 2
'Private Const ALG_SID_RSA_ENTRUST = 3
'Private Const ALG_SID_RSA_PGP = 4
'
'' Some DSS sub-ids
''
'Private Const ALG_SID_DSS_ANY = 0
'Private Const ALG_SID_DSS_PKCS = 1
'Private Const ALG_SID_DSS_DMS = 2
'
'' Block cipher sub ids
'' DES sub_ids
'Private Const ALG_SID_DES = 1
'Private Const ALG_SID_3DES = 3
'Private Const ALG_SID_DESX = 4
'Private Const ALG_SID_IDEA = 5
'Private Const ALG_SID_CAST = 6
'Private Const ALG_SID_SAFERSK64 = 7
'Private Const ALG_SID_SAFERSK128 = 8
'Private Const ALG_SID_3DES_112 = 9
'Private Const ALG_SID_CYLINK_MEK = 12
'Private Const ALG_SID_RC5 = 13
'
'' Fortezza sub-ids
'Private Const ALG_SID_SKIPJACK = 10
'Private Const ALG_SID_TEK = 11
'
'' KP_MODE
'Private Const CRYPT_MODE_CBCI = 6                        ' ANSI CBC Interleaved
'Private Const CRYPT_MODE_CFBP = 7                        ' ANSI CFB Pipelined
'Private Const CRYPT_MODE_OFBP = 8                        ' ANSI OFB Pipelined
'Private Const CRYPT_MODE_CBCOFM = 9                      ' ANSI CBC + OF Masking
'Private Const CRYPT_MODE_CBCOFMI = 10                    ' ANSI CBC + OFM Interleaved

' RC2 sub-ids
Private Const ALG_SID_RC2 = 2

' Stream cipher sub-ids
Private Const ALG_SID_RC4 = 1
Private Const ALG_SID_SEAL = 2

' Diffie-Hellman sub-ids
Private Const ALG_SID_DH_SANDF = 1
Private Const ALG_SID_DH_EPHEM = 2
Private Const ALG_SID_AGREED_KEY_ANY = 3
Private Const ALG_SID_KEA = 4

' Hash sub ids
Private Const ALG_SID_MD2 = 1
Private Const ALG_SID_MD4 = 2
Private Const ALG_SID_MD5 = 3
Private Const ALG_SID_SHA = 4
Private Const ALG_SID_SHA1 = 4
Private Const ALG_SID_MAC = 5
Private Const ALG_SID_RIPEMD = 6
Private Const ALG_SID_RIPEMD160 = 7
Private Const ALG_SID_SSL3SHAMD5 = 8
Private Const ALG_SID_HMAC = 9

'' secure channel sub ids
'Private Const ALG_SID_SSL3_MASTER = 1
'Private Const ALG_SID_SCHANNEL_MASTER_HASH = 2
'Private Const ALG_SID_SCHANNEL_MAC_KEY = 3
'Private Const ALG_SID_PCT1_MASTER = 4
'Private Const ALG_SID_SSL2_MASTER = 5
'Private Const ALG_SID_TLS1_MASTER = 6
'Private Const ALG_SID_SCHANNEL_ENC_KEY = 7
'
'' Our silly example sub-id
'Private Const ALG_SID_EXAMPLE = 80

'' algorithm identifier definitions
'Private Const CALG_MD2 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MD2)
'Private Const CALG_MD4 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MD4)
Private Const CALG_MD5 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MD5)
'Private Const CALG_SHA = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_SHA)
'Private Const CALG_SHA1 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_SHA1)
'Private Const CALG_MAC = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_MAC)
'Private Const CALG_RSA_SIGN = ((ALG_CLASS_SIGNATURE Or ALG_TYPE_RSA) Or ALG_SID_RSA_ANY)
'Private Const CALG_DSS_SIGN = ((ALG_CLASS_SIGNATURE Or ALG_TYPE_DSS) Or ALG_SID_DSS_ANY)
'Private Const CALG_RSA_KEYX = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_RSA) Or ALG_SID_RSA_ANY)
'Private Const CALG_DES = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_DES)
'Private Const CALG_3DES_112 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_3DES_112)
'Private Const CALG_3DES = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_3DES)
Private Const CALG_RC2 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_RC2)
'Private Const CALG_RC4 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_STREAM) Or ALG_SID_RC4)
'Private Const CALG_SEAL = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_STREAM) Or ALG_SID_SEAL)
'Private Const CALG_DH_SF = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_DH) Or ALG_SID_DH_SANDF)
'Private Const CALG_DH_EPHEM = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_DH) Or ALG_SID_DH_EPHEM)
'Private Const CALG_AGREEDKEY_ANY = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_DH) Or ALG_SID_AGREED_KEY_ANY)
'Private Const CALG_KEA_KEYX = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_DH) Or ALG_SID_KEA)
'Private Const CALG_HUGHES_MD5 = ((ALG_CLASS_KEY_EXCHANGE Or ALG_TYPE_ANY) Or ALG_SID_MD5)
'Private Const CALG_SKIPJACK = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_SKIPJACK)
'Private Const CALG_TEK = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_TEK)
'Private Const CALG_CYLINK_MEK = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_CYLINK_MEK)
Private Const CALG_SSL3_SHAMD5 = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_SSL3SHAMD5)
'Private Const CALG_SSL3_MASTER = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_SSL3_MASTER)
'Private Const CALG_SCHANNEL_MASTER_HASH = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_SCHANNEL_MASTER_HASH)
'Private Const CALG_SCHANNEL_MAC_KEY = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_SCHANNEL_MAC_KEY)
'Private Const CALG_SCHANNEL_ENC_KEY = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_SCHANNEL_ENC_KEY)
'Private Const CALG_PCT1_MASTER = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_PCT1_MASTER)
'Private Const CALG_SSL2_MASTER = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_SSL2_MASTER)
'Private Const CALG_TLS1_MASTER = ((ALG_CLASS_MSG_ENCRYPT Or ALG_TYPE_SECURECHANNEL) Or ALG_SID_TLS1_MASTER)
'Private Const CALG_RC5 = ((ALG_CLASS_DATA_ENCRYPT Or ALG_TYPE_BLOCK) Or ALG_SID_RC5)
'Private Const CALG_HMAC = ((ALG_CLASS_HASH Or ALG_TYPE_ANY) Or ALG_SID_HMAC)

' dwFlags definitions for CryptAcquireContext
Private Const CRYPT_VERIFYCONTEXT = &HF0000000
Private Const CRYPT_NEWKEYSET = &H8
Private Const CRYPT_DELETEKEYSET = &H10
Private Const CRYPT_MACHINE_KEYSET = &H20

' dwFlag definitions for CryptGenKey
Private Const CRYPT_EXPORTABLE = &H1
Private Const CRYPT_USER_PROTECTED = &H2
Private Const CRYPT_CREATE_SALT = &H4
Private Const CRYPT_UPDATE_KEY = &H8
Private Const CRYPT_NO_SALT = &H10
Private Const CRYPT_PREGEN = &H40
Private Const CRYPT_RECIPIENT = &H10
Private Const CRYPT_INITIATOR = &H40
Private Const CRYPT_ONLINE = &H80
Private Const CRYPT_SF = &H100
Private Const CRYPT_CREATE_IV = &H200
Private Const CRYPT_KEK = &H400
Private Const CRYPT_DATA_KEY = &H800

' dwFlags definitions for CryptDeriveKey
Private Const CRYPT_SERVER = &H400

Private Const KEY_LENGTH_MASK = &HFFFF0000

'' dwFlag definitions for CryptExportKey
'Private Const CRYPT_Y_ONLY = &H1
'Private Const CRYPT_SSL2_SLUMMING = &H2

'' dwFlags definitions for CryptHashSessionKey
'Private Const CRYPT_LITTLE_ENDIAN = &H1

' dwFlag definitions for CryptSetProviderEx and CryptGetDefaultProvider
Private Const CRYPT_MACHINE_DEFAULT = &H1
Private Const CRYPT_USER_DEFAULT = &H2
Private Const CRYPT_DELETE_DEFAULT = &H4

' exported key blob definitions
Private Const SIMPLEBLOB = &H1
Private Const PUBLICKEYBLOB = &H6
Private Const PRIVATEKEYBLOB = &H7
Private Const PLAINTEXTKEYBLOB = &H8

Private Const AT_KEYEXCHANGE = 1
Private Const AT_SIGNATURE = 2

'Private Const CRYPT_USERDATA = 1
'
'' dwParam
'Private Const KP_IV = 1                          ' Initialization vector
'Private Const KP_SALT = 2                        ' Salt value
'Private Const KP_PADDING = 3                     ' Padding values
'Private Const KP_MODE = 4                        ' Mode of the cipher
'Private Const KP_MODE_BITS = 5                   ' Number of bits to feedback
'Private Const KP_PERMISSIONS = 6                 ' Key permissions DWORD
'Private Const KP_ALGID = 7                       ' Key algorithm
'Private Const KP_BLOCKLEN = 8                    ' Block size of the cipher
'Private Const KP_KEYLEN = 9                      ' Length of key in bits
'Private Const KP_SALT_EX = 10                    ' Length of salt in bytes
'Private Const KP_P = 11                          ' DSS/Diffie-Hellman P value
'Private Const KP_G = 12                          ' DSS/Diffie-Hellman G value
'Private Const KP_Q = 13                          ' DSS Q value
'Private Const KP_X = 14                          ' Diffie-Hellman X value
'Private Const KP_Y = 15                          ' Y value
'Private Const KP_RA = 16                         ' Fortezza RA value
'Private Const KP_RB = 17                         ' Fortezza RB value
'Private Const KP_INFO = 18                       ' for putting information into an RSA envelope
'Private Const KP_EFFECTIVE_KEYLEN = 19           ' setting and getting RC2 effective key length
'Private Const KP_SCHANNEL_ALG = 20               ' for setting the Secure Channel algorithms
'Private Const KP_CLIENT_RANDOM = 21              ' for setting the Secure Channel client random data
'Private Const KP_SERVER_RANDOM = 22              ' for setting the Secure Channel server random data
'Private Const KP_RP = 23
'Private Const KP_PRECOMP_MD5 = 24
'Private Const KP_PRECOMP_SHA = 25
'Private Const KP_CERTIFICATE = 26                ' for setting Secure Channel certificate data (PCT1)
'Private Const KP_CLEAR_KEY = 27                  ' for setting Secure Channel clear key data (PCT1)
'Private Const KP_PUB_EX_LEN = 28
'Private Const KP_PUB_EX_VAL = 29
'
'' KP_PADDING
'Private Const PKCS5_PADDING = 1                  ' PKCS 5 (sec 6.2) padding method
'Private Const RANDOM_PADDING = 2
'Private Const ZERO_PADDING = 3
'
'' KP_MODE
'Private Const CRYPT_MODE_CBC = 1                 ' Cipher block chaining
'Private Const CRYPT_MODE_ECB = 2                 ' Electronic code book
'Private Const CRYPT_MODE_OFB = 3                 ' Output feedback mode
'Private Const CRYPT_MODE_CFB = 4                 ' Cipher feedback mode
'Private Const CRYPT_MODE_CTS = 5                 ' Ciphertext stealing mode
'
'' KP_PERMISSIONS
'Private Const CRYPT_ENCRYPT = &H1                ' Allow encryption
'Private Const CRYPT_DECRYPT = &H2                ' Allow decryption
'Private Const CRYPT_EXPORT = &H4                 ' Allow key to be exported
'Private Const CRYPT_READ = &H8                   ' Allow parameters to be read
'Private Const CRYPT_WRITE = &H10                 ' Allow parameters to be set
'Private Const CRYPT_MAC = &H20                   ' Allow MACs to be used with key
'Private Const CRYPT_EXPORT_KEY = &H40            ' Allow key to be used for exporting keys
'Private Const CRYPT_IMPORT_KEY = &H80            ' Allow key to be used for importing keys
'
Private Const HP_ALGID = &H1                     ' Hash algorithm
Private Const HP_HASHVAL = &H2                   ' Hash value
Private Const HP_HASHSIZE = &H4                  ' Hash value size
Private Const HP_HMAC_INFO = &H5                 ' information for creating an HMAC

Private Const CRYPT_FAILED = 0
Private Const CRYPT_SUCCEED = -1

''
'' CryptGetProvParam
''
'Private Const PP_ENUMALGS = 1
'Private Const PP_ENUMCONTAINERS = 2
'Private Const PP_IMPTYPE = 3
'Private Const PP_NAME = 4
'Private Const PP_VERSION = 5
'Private Const PP_CONTAINER = 6
'Private Const PP_CHANGE_PASSWORD = 7
'Private Const PP_KEYSET_SEC_DESCR = 8            ' get/set security descriptor of keyset
'Private Const PP_CERTCHAIN = 9                   ' for retrieving certificates from tokens
'Private Const PP_KEY_TYPE_SUBTYPE = 10
'Private Const PP_PROVTYPE = 16
'Private Const PP_KEYSTORAGE = 17
'Private Const PP_APPLI_CERT = 18
'Private Const PP_SYM_KEYSIZE = 19
'Private Const PP_SESSION_KEYSIZE = 20
'Private Const PP_UI_PROMPT = 21
'Private Const PP_ENUMALGS_EX = 22
'
'Private Const CRYPT_FIRST = 1
'Private Const CRYPT_NEXT = 2
'
'Private Const CRYPT_IMPL_HARDWARE = 1
'Private Const CRYPT_IMPL_SOFTWARE = 2
'Private Const CRYPT_IMPL_MIXED = 3
'Private Const CRYPT_IMPL_UNKNOWN = 4
'
'' key storage flags
'Private Const CRYPT_SEC_DESCR = &H1
'Private Const CRYPT_PSTORE = &H2
'Private Const CRYPT_UI_PROMPT = &H4
'
'' protocol flags
'Private Const CRYPT_FLAG_PCT1 = &H1
'Private Const CRYPT_FLAG_SSL2 = &H2
'Private Const CRYPT_FLAG_SSL3 = &H4
'Private Const CRYPT_FLAG_TLS1 = &H8
'
''
'' CryptSetProvParam
''
'Private Const PP_CLIENT_HWND = 1
'Private Const PP_CONTEXT_INFO = 11
'Private Const PP_KEYEXCHANGE_KEYSIZE = 12
'Private Const PP_SIGNATURE_KEYSIZE = 13
'Private Const PP_KEYEXCHANGE_ALG = 14
'Private Const PP_SIGNATURE_ALG = 15
'Private Const PP_DELETEKEY = 24

Private Const PROV_RSA_FULL = 1
Private Const PROV_RSA_SIG = 2
Private Const PROV_DSS = 3
Private Const PROV_FORTEZZA = 4
Private Const PROV_MS_EXCHANGE = 5
Private Const PROV_SSL = 6
Private Const PROV_RSA_SCHANNEL = 12
Private Const PROV_DSS_DH = 13
Private Const PROV_EC_ECDSA_SIG = 14
Private Const PROV_EC_ECNRA_SIG = 15
Private Const PROV_EC_ECDSA_FULL = 16
Private Const PROV_EC_ECNRA_FULL = 17
Private Const PROV_SPYRUS_LYNKS = 20

''
'' STT defined Providers
''
'Private Const PROV_STT_MER = 7
'Private Const PROV_STT_ACQ = 8
'Private Const PROV_STT_BRND = 9
'Private Const PROV_STT_ROOT = 10
'Private Const PROV_STT_ISS = 11

'
' Provider friendly names
'
Private Const MS_DEF_PROV = "Microsoft Base Cryptographic Provider v1.0"
Private Const MS_ENHANCED_PROV = "Microsoft Enhanced Cryptographic Provider v1.0"
Private Const MS_DEF_RSA_SIG_PROV = "Microsoft RSA Signature Cryptographic Provider"
Private Const MS_DEF_RSA_SCHANNEL_PROV = "Microsoft Base RSA SChannel Cryptographic Provider"
Private Const MS_ENHANCED_RSA_SCHANNEL_PROV = "Microsoft Enhanced RSA SChannel Cryptographic Provider"
Private Const MS_DEF_DSS_PROV = "Microsoft Base DSS Cryptographic Provider"
Private Const MS_DEF_DSS_DH_PROV = "Microsoft Base DSS and Diffie-Hellman Cryptographic Provider"

'Private Const MAXUIDLEN = 64
'
'Private Const CUR_BLOB_VERSION = 2
'
'' uses of algortihms for SCHANNEL_ALG structure
'Private Const SCHANNEL_MAC_KEY = &H0
'Private Const SCHANNEL_ENC_KEY = &H1

'+-------------------------------------------------------------------------
'  Certificate name types
'--------------------------------------------------------------------------
Private Const CERT_NAME_EMAIL_TYPE = 1
Private Const CERT_NAME_RDN_TYPE = 2
Private Const CERT_NAME_ATTR_TYPE = 3
Private Const CERT_NAME_SIMPLE_DISPLAY_TYPE = 4
Private Const CERT_NAME_FRIENDLY_DISPLAY_TYPE = 5

Private Const CERT_SYSTEM_STORE_MASK = &HFFFF0000

'+-------------------------------------------------------------------------
'  Certificate, CRL and CTL property IDs
'
'  See CertSetCertificateContextProperty or CertGetCertificateContextProperty
'  for usage information.
'--------------------------------------------------------------------------
Private Const CERT_KEY_PROV_HANDLE_PROP_ID = 1
Private Const CERT_KEY_PROV_INFO_PROP_ID = 2
Private Const CERT_SHA1_HASH_PROP_ID = 3
Private Const CERT_MD5_HASH_PROP_ID = 4
Private Const CERT_HASH_PROP_ID = CERT_SHA1_HASH_PROP_ID
Private Const CERT_KEY_CONTEXT_PROP_ID = 5
Private Const CERT_KEY_SPEC_PROP_ID = 6
Private Const CERT_IE30_RESERVED_PROP_ID = 7
Private Const CERT_PUBKEY_HASH_RESERVED_PROP_ID = 8
Private Const CERT_ENHKEY_USAGE_PROP_ID = 9
Private Const CERT_CTL_USAGE_PROP_ID = CERT_ENHKEY_USAGE_PROP_ID
Private Const CERT_NEXT_UPDATE_LOCATION_PROP_ID = 10
Private Const CERT_FRIENDLY_NAME_PROP_ID = 11
Private Const CERT_PVK_FILE_PROP_ID = 12
Private Const CERT_DESCRIPTION_PROP_ID = 13
Private Const CERT_ACCESS_STATE_PROP_ID = 14
Private Const CERT_SIGNATURE_HASH_PROP_ID = 15
Private Const CERT_SMART_CARD_DATA_PROP_ID = 16
Private Const CERT_EFS_PROP_ID = 17
Private Const CERT_FORTEZZA_DATA_PROP_ID = 18
Private Const CERT_ARCHIVED_PROP_ID = 19
Private Const CERT_KEY_IDENTIFIER_PROP_ID = 20
Private Const CERT_AUTO_ENROLL_PROP_ID = 21
Private Const CERT_PUBKEY_ALG_PARA_PROP_ID = 22

Private Const CERT_FIRST_RESERVED_PROP_ID = 23
' Note, 32 - 35 are reserved for the CERT, CRL, CTL and KeyId file element IDs.

Private Const CERT_LAST_RESERVED_PROP_ID = &H7FFF
Private Const CERT_FIRST_USER_PROP_ID = &H8000
Private Const CERT_LAST_USER_PROP_ID = &HFFFF

'
' Certificate Status
'
Const CA_DISP_INCOMPLETE = 0
Const CA_DISP_ERROR = &H1
Const CA_DISP_REVOKED = &H2
Const CA_DISP_VALID = &H3
Const CA_DISP_INVALID = &H4
Const CA_DISP_UNDER_SUBMISSION = &H5

'+-------------------------------------------------------------------------
'  X509_CRL_REASON_CODE
'  szOID_CRL_REASON_CODE
'
'  pvStructInfo points to an int which can be set to one of the following
'  enumerated values:
'--------------------------------------------------------------------------
Const CRL_REASON_UNSPECIFIED = 0
Const CRL_REASON_KEY_COMPROMISE = 1
Const CRL_REASON_CA_COMPROMISE = 2
Const CRL_REASON_AFFILIATION_CHANGED = 3
Const CRL_REASON_SUPERSEDED = 4
Const CRL_REASON_CESSATION_OF_OPERATION = 5
Const CRL_REASON_CERTIFICATE_HOLD = 6
Const CRL_REASON_REMOVE_FROM_CRL = 8

'
' Issued Certificate Format
'
Const CR_OUT_BASE64HEADER = 0
Const CR_OUT_BASE64 = &H1
Const CR_OUT_BINARY = &H2
Const CR_OUT_ENCODEMASK = &HFF
Const CR_OUT_CHAIN = &H100

Const CRYPT_ASN_ENCODING = &H1
Const CRYPT_NDR_ENCODING = &H2
Const X509_ASN_ENCODING = &H1
Const X509_NDR_ENCODING = &H2
Const PKCS_7_ASN_ENCODING = &H10000
Const PKCS_7_NDR_ENCODING = &H20000

'+-------------------------------------------------------------------------
' Add certificate/CRL, encoded, context or element disposition values.
'--------------------------------------------------------------------------
Const CERT_STORE_ADD_NEW = 1
Const CERT_STORE_ADD_USE_EXISTING = 2
Const CERT_STORE_ADD_REPLACE_EXISTING = 3
Const CERT_STORE_ADD_ALWAYS = 4
Const CERT_STORE_ADD_REPLACE_EXISTING_INHERIT_PROPERTIES = 5
Const CERT_STORE_ADD_NEWER = 6
Const CERT_STORE_ADD_NEWER_INHERIT_PROPERTIES = 7

Const CERT_FIND_ANY = 0
Const CERT_FIND_EXISTING = &HD0000
Const CERT_FIND_ISSUER_OF = &HC0000
Const CERT_FIND_ISSUER_STR = &H70004
Const CERT_FIND_KEY_SPEC = &H90000
Const CERT_FIND_PROPERTY = &H50000
Const CERT_FIND_SUBJECT_STR = &H70007

' Following are the definitions of various algorithm object identifiers
' RSA
'Const szOID_RSA = "1.2.840.113549"
'Const szOID_PKCS = "1.2.840.113549.1"
'Const szOID_RSA_HASH = "1.2.840.113549.2"
'Const szOID_RSA_ENCRYPT = "1.2.840.113549.3"
'
'Const szOID_PKCS_1 = "1.2.840.113549.1.1"
'Const szOID_PKCS_2 = "1.2.840.113549.1.2"
'Const szOID_PKCS_3 = "1.2.840.113549.1.3"
'Const szOID_PKCS_4 = "1.2.840.113549.1.4"
'Const szOID_PKCS_5 = "1.2.840.113549.1.5"
'Const szOID_PKCS_6 = "1.2.840.113549.1.6"
'Const szOID_PKCS_7 = "1.2.840.113549.1.7"
'Const szOID_PKCS_8 = "1.2.840.113549.1.8"
'Const szOID_PKCS_9 = "1.2.840.113549.1.9"
'Const szOID_PKCS_10 = "1.2.840.113549.1.10"
'
'Const szOID_RSA_RSA = "1.2.840.113549.1.1.1"
Const szOID_RSA_MD2RSA = "1.2.840.113549.1.1.2"
'Const szOID_RSA_MD4RSA = "1.2.840.113549.1.1.3"
Const szOID_RSA_MD5RSA = "1.2.840.113549.1.1.4"
'Const szOID_RSA_SHA1RSA = "1.2.840.113549.1.1.5"
'Const szOID_RSA_SETOAEP_RSA = "1.2.840.113549.1.1.6"
'
'Const szOID_RSA_data = "1.2.840.113549.1.7.1"
'Const szOID_RSA_signedData = "1.2.840.113549.1.7.2"
'Const szOID_RSA_envelopedData = "1.2.840.113549.1.7.3"
'Const szOID_RSA_signEnvData = "1.2.840.113549.1.7.4"
'Const szOID_RSA_digestedData = "1.2.840.113549.1.7.5"
'Const szOID_RSA_hashedData = "1.2.840.113549.1.7.5"
'Const szOID_RSA_encryptedData = "1.2.840.113549.1.7.6"
'
'Const szOID_RSA_emailAddr = "1.2.840.113549.1.9.1"
'Const szOID_RSA_unstructName = "1.2.840.113549.1.9.2"
'Const szOID_RSA_contentType = "1.2.840.113549.1.9.3"
'Const szOID_RSA_messageDigest = "1.2.840.113549.1.9.4"
'Const szOID_RSA_signingTime = "1.2.840.113549.1.9.5"
'Const szOID_RSA_counterSign = "1.2.840.113549.1.9.6"
'Const szOID_RSA_challengePwd = "1.2.840.113549.1.9.7"
'Const szOID_RSA_unstructAddr = "1.2.840.113549.1.9.8"
'Const szOID_RSA_extCertAttrs = "1.2.840.113549.1.9.9"
'Const szOID_RSA_SMIMECapabilities = "1.2.840.113549.1.9.15"
'Const szOID_RSA_preferSignedData = "1.2.840.113549.1.9.15.1"
'
'Const szOID_RSA_MD2 = "1.2.840.113549.2.2"
'Const szOID_RSA_MD4 = "1.2.840.113549.2.4"
Const szOID_RSA_MD5 = "1.2.840.113549.2.5"
'
'Const szOID_RSA_RC2CBC = "1.2.840.113549.3.2"
Const szOID_RSA_RC4 = "1.2.840.113549.3.4"
'Const szOID_RSA_DES_EDE3_CBC = "1.2.840.113549.3.7"
'Const szOID_RSA_RC5_CBCPad = "1.2.840.113549.3.9"
'
'' ITU-T UsefulDefinitions
'Const szOID_DS = "2.5"
'Const szOID_DSALG = "2.5.8"
'Const szOID_DSALG_CRPT = "2.5.8.1"
'Const szOID_DSALG_HASH = "2.5.8.2"
'Const szOID_DSALG_SIGN = "2.5.8.3"
'Const szOID_DSALG_RSA = "2.5.8.1.1"
'' NIST OSE Implementors' Workshop (OIW)
'' http:'nemo.ncsl.nist.gov/oiw/agreements/stable/OSI/12s_9506.w51
'' http:'nemo.ncsl.nist.gov/oiw/agreements/working/OSI/12w_9503.w51
'Const szOID_OIW = "1.3.14"
'' NIST OSE Implementors' Workshop (OIW) Security SIG algorithm identifiers
'Const szOID_OIWSEC = "1.3.14.3.2"
'Const szOID_OIWSEC_md4RSA = "1.3.14.3.2.2"
'Const szOID_OIWSEC_md5RSA = "1.3.14.3.2.3"
'Const szOID_OIWSEC_md4RSA2 = "1.3.14.3.2.4"
'Const szOID_OIWSEC_desECB = "1.3.14.3.2.6"
'Const szOID_OIWSEC_desCBC = "1.3.14.3.2.7"
'Const szOID_OIWSEC_desOFB = "1.3.14.3.2.8"
'Const szOID_OIWSEC_desCFB = "1.3.14.3.2.9"
'Const szOID_OIWSEC_desMAC = "1.3.14.3.2.10"
'Const szOID_OIWSEC_rsaSign = "1.3.14.3.2.11"
'Const szOID_OIWSEC_dsa = "1.3.14.3.2.12"
'Const szOID_OIWSEC_shaDSA = "1.3.14.3.2.13"
'Const szOID_OIWSEC_mdc2RSA = "1.3.14.3.2.14"
'Const szOID_OIWSEC_shaRSA = "1.3.14.3.2.15"
'Const szOID_OIWSEC_dhCommMod = "1.3.14.3.2.16"
'Const szOID_OIWSEC_desEDE = "1.3.14.3.2.17"
'Const szOID_OIWSEC_sha = "1.3.14.3.2.18"
'Const szOID_OIWSEC_mdc2 = "1.3.14.3.2.19"
'Const szOID_OIWSEC_dsaComm = "1.3.14.3.2.20"
'Const szOID_OIWSEC_dsaCommSHA = "1.3.14.3.2.21"
'Const szOID_OIWSEC_rsaXchg = "1.3.14.3.2.22"
'Const szOID_OIWSEC_keyHashSeal = "1.3.14.3.2.23"
'Const szOID_OIWSEC_md2RSASign = "1.3.14.3.2.24"
'Const szOID_OIWSEC_md5RSASign = "1.3.14.3.2.25"
'Const szOID_OIWSEC_sha1 = "1.3.14.3.2.26"
'Const szOID_OIWSEC_dsaSHA1 = "1.3.14.3.2.27"
'Const szOID_OIWSEC_dsaCommSHA1 = "1.3.14.3.2.28"
'Const szOID_OIWSEC_sha1RSASign = "1.3.14.3.2.29"
'' NIST OSE Implementors' Workshop (OIW) Directory SIG algorithm identifiers
'Const szOID_OIWDIR = "1.3.14.7.2"
'Const szOID_OIWDIR_CRPT = "1.3.14.7.2.1"
'Const szOID_OIWDIR_HASH = "1.3.14.7.2.2"
'Const szOID_OIWDIR_SIGN = "1.3.14.7.2.3"
'Const szOID_OIWDIR_md2 = "1.3.14.7.2.2.1"
'Const szOID_OIWDIR_md2RSA = "1.3.14.7.2.3.1"
'
'
'' INFOSEC Algorithms
'' joint-iso-ccitt(2) country(16) us(840) organization(1) us-government(101) dod(2) id-infosec(1)
'Const szOID_INFOSEC = "2.16.840.1.101.2.1"
'Const szOID_INFOSEC_sdnsSignature = "2.16.840.1.101.2.1.1.1"
'Const szOID_INFOSEC_mosaicSignature = "2.16.840.1.101.2.1.1.2"
'Const szOID_INFOSEC_sdnsConfidentiality = "2.16.840.1.101.2.1.1.3"
'Const szOID_INFOSEC_mosaicConfidentiality = "2.16.840.1.101.2.1.1.4"
'Const szOID_INFOSEC_sdnsIntegrity = "2.16.840.1.101.2.1.1.5"
'Const szOID_INFOSEC_mosaicIntegrity = "2.16.840.1.101.2.1.1.6"
'Const szOID_INFOSEC_sdnsTokenProtection = "2.16.840.1.101.2.1.1.7"
'Const szOID_INFOSEC_mosaicTokenProtection = "2.16.840.1.101.2.1.1.8"
'Const szOID_INFOSEC_sdnsKeyManagement = "2.16.840.1.101.2.1.1.9"
'Const szOID_INFOSEC_mosaicKeyManagement = "2.16.840.1.101.2.1.1.10"
'Const szOID_INFOSEC_sdnsKMandSig = "2.16.840.1.101.2.1.1.11"
'Const szOID_INFOSEC_mosaicKMandSig = "2.16.840.1.101.2.1.1.12"
'Const szOID_INFOSEC_SuiteASignature = "2.16.840.1.101.2.1.1.13"
'Const szOID_INFOSEC_SuiteAConfidentiality = "2.16.840.1.101.2.1.1.14"
'Const szOID_INFOSEC_SuiteAIntegrity = "2.16.840.1.101.2.1.1.15"
'Const szOID_INFOSEC_SuiteATokenProtection = "2.16.840.1.101.2.1.1.16"
'Const szOID_INFOSEC_SuiteAKeyManagement = "2.16.840.1.101.2.1.1.17"
'Const szOID_INFOSEC_SuiteAKMandSig = "2.16.840.1.101.2.1.1.18"
'Const szOID_INFOSEC_mosaicUpdatedSig = "2.16.840.1.101.2.1.1.19"
'Const szOID_INFOSEC_mosaicKMandUpdSig = "2.16.840.1.101.2.1.1.20"
'Const szOID_INFOSEC_mosaicUpdatedInteg = "2.16.840.1.101.2.1.1.21"

Private Type CRYPT_KEY_PROV_PARAM
    dwParam As Long
    pbData As Long
    cbData As Long
    dwFlags As Long
End Type

Private Type CRYPT_KEY_PROV_INFO
    pwszContainerName As Long
    pwszProvName As Long
    dwProvType As Long
    dwFlags As Long
    cProvParam As Long
    rgProvParam As Long
    dwKeySpec As Long
End Type

Private Type CERT_KEY_CONTEXT
    cbSize As Long           ' sizeof(CERT_KEY_CONTEXT)
    hCryptProv As Long
    dwKeySpec As Long
End Type

Private Type FILETIME
        dwLowDateTime As Long
        dwHighDateTime As Long
End Type

Private Type CRYPT_INTEGER_BLOB
    cbData As Long
    pbData As Long  ' Pointer to Byte Array
End Type

Private Type CRYPT_ALGORITHM_IDENTIFIER
'    pszObjId As Long    ' Pointer to String
    pszObjId As String    ' Pointer to String
    Parameters As CRYPT_INTEGER_BLOB
End Type

Private Type CRYPT_BIT_BLOB
    cbData As Long
    pbData As Long  ' Pointer to Byte Array
    cUnusedBits As Long
End Type

Private Type CERT_PUBLIC_KEY_INFO
    Algorithm As CRYPT_ALGORITHM_IDENTIFIER
    PublicKey As CRYPT_BIT_BLOB
End Type

Private Type CERT_INFO
    dwVersion As Long
    SerialNumber As CRYPT_INTEGER_BLOB
    SignatureAlgorithm As CRYPT_ALGORITHM_IDENTIFIER
    Issuer As CRYPT_INTEGER_BLOB
    NotBefore As FILETIME
    NotAfter As FILETIME
    Subject As CRYPT_INTEGER_BLOB
    SubjectPublicKeyInfo As CERT_PUBLIC_KEY_INFO
    IssuerUniqueId As CRYPT_BIT_BLOB
    SubjectUniqueId As CRYPT_BIT_BLOB
    cExtension As Long
    rgExtension As Long     ' Pointer to CERT_EXTENSION
End Type

Private Type CERT_CONTEXT
    dwCertEncodingType As Long
    pbCertEncoded As Long   ' Pointer to Byte Array
    cbCertEncoded As Long
    pCertInfo As Long       ' Pointer to CERT_INFO
    hCertStore As Long
End Type

Private Type CRL_CONTEXT
    dwCertEncodingType As Long
    pbCrlEncoded As Long
    cbCrlEncoded As Long
    pCrlInfo As Long        ' Pointer to CRL_INFO
    hCertStore As Long
End Type

Private Type CRYPT_SIGN_MESSAGE_PARA
    cbSize As Long
    dwMsgEncodingType As Long
    pSigningCert As Long        ' Pointer to signing certificate context
    HashAlgorithm As CRYPT_ALGORITHM_IDENTIFIER
    pvHashAuxInfo As Long
    cMsgCert As Long
    rgpMsgCert As Long          ' Pointer to array of certificate context pointers
    cMsgCrl As Long
    rgpMsgCrl As Long           ' Pointer to array of CRL context pointers
    cAuthAttr As Long
    rgAuthAttr As Long          ' Pointer to CRYPT_ATTRIBUTE type
    cUnauthAttr As Long
    rgUnauthAttr As Long        ' Pointer to CRYPT_ATTRIBUTE type
    dwFlags As Long
    dwInnerContentType As Long
End Type

Private Type CRYPT_VERIFY_MESSAGE_PARA
    cbSize As Long
    dwMsgAndCertEncodingType As Long
    hCryptProv As Long
    pfnGetSignerCertificate As Long     ' Pointer to callback function
    pvGetArg As Long                    ' void Pointer
End Type

Private Type CRYPT_ENCRYPT_MESSAGE_PARA
    cbSize As Long
    dwMsgEncodingType As Long
    hCryptProv As Long
    ContentEncryptionAlgorithm As CRYPT_ALGORITHM_IDENTIFIER
    pvEncryptionAuxInfo As Long    ' void Pointer
    dwFlags As Long
    dwInnerContentType As Long
End Type

Private Type CRYPT_DECRYPT_MESSAGE_PARA
    cbSize As Long
    dwMsgAndCertEncodingType As Long
    cCertStore As Long
    rghCertStore As Long        ' Pointer to Certificate Store handle
End Type

'--- WinCrypt API Declarations
Private Declare Function CryptAcquireContext Lib "advapi32.dll" Alias "CryptAcquireContextA" ( _
    phProv As Long, _
    ByVal pszContainer As String, _
    ByVal pszProvider As String, _
    ByVal dwProvType As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptReleaseContext Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptGenKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptDeriveKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hBaseData As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptDestroyKey Lib "advapi32.dll" ( _
    ByVal hKey As Long) As Long

Private Declare Function CryptGetHashParam Lib "advapi32.dll" ( _
    ByVal hHash As Long, _
    ByVal dwParam As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptGetUserKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal dwKeySpec As Long, _
    phUserKey As Long) As Long

Private Declare Function CryptExportKey Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hExpKey As Long, _
    ByVal dwBlobType As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long) As Long

Private Declare Function CryptImportKey Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal pbData As String, _
    ByVal dwDataLen As Long, _
    ByVal hPubKey As Long, _
    ByVal dwFlags As Long, _
    phKey As Long) As Long

Private Declare Function CryptEncrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long, _
    ByVal dwBufLen As Long) As Long

Private Declare Function CryptDecrypt Lib "advapi32.dll" ( _
    ByVal hKey As Long, _
    ByVal hHash As Long, _
    ByVal Final As Long, _
    ByVal dwFlags As Long, _
    ByVal pbData As String, _
    pdwDataLen As Long) As Long

Private Declare Function CryptCreateHash Lib "advapi32.dll" ( _
    ByVal hProv As Long, _
    ByVal Algid As Long, _
    ByVal hKey As Long, _
    ByVal dwFlags As Long, _
    phHash As Long) As Long

Private Declare Function CryptHashData Lib "advapi32.dll" ( _
    ByVal hHash As Long, _
    ByVal pbData As String, _
    ByVal dwDataLen As Long, _
    ByVal dwFlags As Long) As Long

Private Declare Function CryptDestroyHash Lib "advapi32.dll" ( _
    ByVal hHash As Long) As Long

Private Declare Function CertOpenSystemStore Lib "Crypt32.dll" Alias "CertOpenSystemStoreA" ( _
    ByVal hProv As Long, _
    ByVal szSubsystemProtocol As String _
    ) As Long

Private Declare Function CertEnumCertificatesInStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal pPrevCertContext As Long _
    ) As Long

Private Declare Function CertEnumCertificateContextProperties Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long _
    ) As Long

Private Declare Function CertGetCertificateContextProperty Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByVal pvData As String, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextDWProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As Long, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextKCProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As CERT_KEY_CONTEXT, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertGetCertificateContextKPIProperty Lib "Crypt32.dll" Alias "CertGetCertificateContextProperty" ( _
    ByVal pCertContext As Long, _
    ByVal dwPropId As Long, _
    ByRef pvData As CRYPT_KEY_PROV_INFO, _
    ByRef pcbData As Long _
    ) As Long

Private Declare Function CertFreeCertificateContext Lib "Crypt32.dll" ( _
    ByVal pCertContext As Long _
    ) As Long

Private Declare Function CertCloseStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal dwFlags As Long _
    ) As Long

Private Declare Function CertGetNameString Lib "Crypt32.dll" Alias "CertGetNameStringA" ( _
    ByVal pCertContext As Long, _
    ByVal dwType As Long, _
    ByVal dwFlags As Long, _
    ByRef pvTypePara As Long, _
    ByVal pszNameString As String, _
    ByVal cchNameString As Long _
    ) As Long

Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" ( _
    Destination As Any, _
    ByVal Source As Long, _
    ByVal Length As Long)

Private Declare Function CertAddEncodedCRLToStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal dwCertEncodingType As Long, _
    ByVal pbCrlEncoded As String, _
    ByVal cbCrlEncoded As Long, _
    ByVal dwAddDisposition As Long, _
    ByVal ppCrlContext As Long _
    ) As Long

Private Declare Function CertCreateCRLContext Lib "Crypt32.dll" ( _
    ByVal dwCertEncodingType As Long, _
    ByVal pbCrlEncoded As String, _
    ByVal cbCrlEncoded As Long _
    ) As Long

Private Declare Function CertVerifyCRLRevocation Lib "Crypt32.dll" ( _
    ByVal dwCertEncodingType As Long, _
    ByVal pCertId As Long, _
    ByVal cCrlInfo As Long, _
    ByRef rgpCrlInfo As Any _
    ) As Long

Private Declare Function CertFindCertificateInStore Lib "Crypt32.dll" ( _
    ByVal hCertStore As Long, _
    ByVal dwCertEncodingType As Long, _
    ByVal dwFindFlags As Long, _
    ByVal dwFindType As Long, _
    ByVal pvFindPara As String, _
    ByVal pPrevCertContext As Long _
    ) As Long

Private Declare Function CryptSignMessage Lib "Crypt32.dll" ( _
    pSignPara As CRYPT_SIGN_MESSAGE_PARA, _
    ByVal fDetachedSignature As Long, _
    ByVal cToBeSigned As Long, _
    rgpbToBeSigned As String, _
    rgcbToBeSigned As Long, _
    ByVal pbSignedBlob As String, _
    pcbSignedBlob As Long _
    ) As Long

Private Declare Function CryptVerifyMessageSignature Lib "Crypt32.dll" ( _
    pVerifyPara As CRYPT_VERIFY_MESSAGE_PARA, _
    ByVal dwSignerIndex As Long, _
    ByVal pbSignedBlob As String, _
    ByVal cbSignedBlob As Long, _
    ByVal pbDecoded As String, _
    pcbDecoded As Long, _
    ppSignerCert As Long _
    ) As Long

Private Declare Function CryptSignAndEncryptMessage Lib "Crypt32.dll" ( _
    pSignPara As CRYPT_SIGN_MESSAGE_PARA, _
    pEncryptPara As CRYPT_ENCRYPT_MESSAGE_PARA, _
    ByVal cRecipientCert As Long, _
    rgpRecipientCert As Long, _
    ByVal pbToBeSignedAndEncrypted As String, _
    ByVal cbToBeSignedAndEncrypted As Long, _
    ByVal pbSignedAndEncryptedBlob As String, _
    pcbSignedAndEncryptedBlob As Long _
    ) As Long

Private Declare Function CryptDecryptAndVerifyMessageSignature Lib "Crypt32.dll" ( _
    pDecryptPara As CRYPT_DECRYPT_MESSAGE_PARA, _
    pVerifyPara As CRYPT_VERIFY_MESSAGE_PARA, _
    ByVal dwSignerIndex As Long, _
    ByVal pbEncryptedBlob As String, _
    ByVal cbEncryptedBlob As Long, _
    ByVal pbDecrypted As String, _
    pcbDecrypted As Long, _
    ppXchgCert As Long, _
    ppSignerCert As Long _
    ) As Long

Private Declare Function CryptEncryptMessage Lib "Crypt32.dll" ( _
    pEncryptPara As CRYPT_ENCRYPT_MESSAGE_PARA, _
    ByVal cRecipientCert As Long, _
    ByVal rgpRecipientCert As Long, _
    ByVal pbToBeEncrypted As String, _
    ByVal cbToBeEncrypted As Long, _
    ByVal pbEncryptedBlob As String, _
    pcbEncryptedBlob As Long _
    ) As Long

Private Declare Function CryptDecryptMessage Lib "Crypt32.dll" ( _
    pDecryptPara As CRYPT_DECRYPT_MESSAGE_PARA, _
    ByVal pbEncryptedBlob As String, _
    ByVal cbEncryptedBlob As Long, _
    ByVal pbDecrypted As String, _
    pcbDecrypted As Long, _
    ppXchgCert As Long _
    ) As Long

'constants from WinErr.h
Private Const ERROR_INVALID_HANDLE As Long = 6
Private Const ERROR_INVALID_PARAMETER As Long = 87
Private Const NTE_BAD_KEY As Long = &H80090003
Private Const NTE_BAD_UID As Long = &H80090001
Private Const NTE_NO_KEY As Long = &H8009000D
Private Const NTE_BAD_SIGNATURE As Long = &H80090006
Private Const CRYPT_E_EXISTS As Long = &H80092005
Private Const CRYPT_E_OSS_ERROR As Long = &H80093000
Private Const E_INVALIDARG As Long = &H80070057
Private Const ERROR_MORE_DATA As Long = 234
Private Const CRYPT_E_NO_KEY_PROPERTY As Long = &H8009200B

' Private property buffers
Private m_sPassword As String   ' Password used to create encryption key

Private m_sInBuffer As String   ' Used as an input buffer for all
                                ' data to be encrypted or decrypted

Private m_sOutBuffer As String  ' Used as an output buffer for all
                                ' data that has been encrypted or decrypted

Private m_sExportKey As String  ' Used to publish the exported public
                                ' key that can be sent to the other
                                ' application that is to be sending
                                ' encrypted data to this class

Private m_sImportKey As String  ' Used to import the public key of
                                ' the other application to which this
                                ' application will be sending encrypted
                                ' data

Private m_sSessionKey As String ' Used to publish and import the
                                ' encrypted session key that is used
                                ' to encrypt and decrypt the data to
                                ' be passed between applications
                                
Private m_strServer As String   ' The CA Server name

Private m_strCA As String       ' The CA Name

Private m_sErrorMsg As String   ' Error message string

' Private class-level variables
Private m_lHCryptProv As Long   ' Handle for the cryptographic
                                ' service provider (CSP)

Private m_lHCryptKey As Long    ' Handle for the AT_KEYEXCHANGE
                                ' (Public/Private) key for this
                                ' application

Private m_lHImportKey As Long   ' Handle for the imported public
                                ' key of the application to which
                                ' this application will be sending
                                ' data

Private m_lHSessionKey As Long  ' Session key for encrypting and
                                ' decrypting data

'Private m_lHFileHash As Long    ' Handle to the hash of the encrypted
'                                ' file

Private m_sHashBuffer As String ' Used as a buffer for all the hash value

Private m_iHashLength As Long   ' Used to hold the hash length

Private m_lCRLPointer As Long   ' Used to hold a CRL

Private m_lCertContextPointer As Long   ' Used to hold a pointer to a CERT_CONTEXT

Private m_hCertStore As Long    ' Used to hold a handle to the certificate store

Private m_bUseCRLToVerifyCerts As Boolean

Public Property Get InBuffer() As String
    InBuffer = m_sInBuffer
End Property

Public Property Let InBuffer(vNewValue As String)
    m_sInBuffer = vNewValue
End Property

Public Property Get OutBuffer() As String
    OutBuffer = m_sOutBuffer
End Property

Public Property Get ExportKey() As String
    ExportKey = m_sExportKey
End Property

Public Property Get SessionKey() As String
    SessionKey = m_sSessionKey
End Property

Public Property Let SessionKey(vNewValue As String)
    m_sSessionKey = vNewValue
End Property

Public Property Get ImportKey() As String
    ImportKey = m_sImportKey
End Property

Public Property Let ImportKey(vNewValue As String)
    m_sImportKey = vNewValue
End Property

Public Property Get ErrorMsg() As String
    ErrorMsg = m_sErrorMsg
End Property

Private Sub Class_Initialize()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Initialize
'*
'* Parameters:  None
'*
'* Purpose: Calls the InitUser function to initialize the
'*          class. The initialization process loads the CSP
'*          and Public/Private key.
'*************************************************************
    If Not InitUser Then
        m_sErrorMsg = "Unable to initialize CryptoAPI."
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
    End If
    
    m_bUseCRLToVerifyCerts = False
End Sub

Private Function InitUser() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     InitUser
'*
'* Parameters:  None
'*
'* Purpose: This function loads the default CSP (specified
'*          in the defined constant MS_DEF_PROV), and the
'*          AT_KEYEXCHANGE keyset. If the keyset or key
'*          container does not exist, then this function
'*          creates them.
'*************************************************************
On Error Resume Next
    InitUser = False
    
    Dim lDataSize As Long
    
    Dim lResult As Long
    Dim sResult As String
    Dim sContainer As String
    Dim sProvider As String
    
    '--- Prepare string buffers
    sContainer = vbNullChar
    sProvider = MS_DEF_PROV & vbNullChar
    
    '--- Attempt to acquire a handle to the default key container.
    If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, 0)) Then
    
        '--- Create default key container.
        If Not CBool(CryptAcquireContext(m_lHCryptProv, sContainer, sProvider, PROV_RSA_FULL, CRYPT_NEWKEYSET)) Then
            m_sErrorMsg = "Error creating key container - " & CStr(Err.LastDllError)
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
            
            Exit Function
        End If
        
    End If
    
    '--- Didn't exit early, return TRUE
    InitUser = True

    Exit Function
    
End Function

Public Function ExportPublicKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     ExportPublicKey
'*
'* Parameters:  None
'*
'* Purpose: This function exports the public key from the
'*          AT_KEYEXCHANGE keyset, placing the exported key
'*          into the ExportKey property
'*************************************************************
On Error Resume Next
    ExportPublicKey = False

    Dim lDataSize As Long
    Dim lResult As Long

    '--- Null the target string, so we can get the size of the string
    '--- that will be exported
    lDataSize = 0

    '--- Call CryptExportKey with the nulled string. This will give us
    '--- the required string size that will be needed to hold the exported
    '--- key
    lResult = CryptExportKey(m_lHCryptKey, 0, PUBLICKEYBLOB, 0, vbNullString, lDataSize)

    '--- Size the target string for the size specified, filling it with
    '--- null characters
    m_sExportKey = String(lDataSize, vbNullChar)

    '--- Export the public key
    If Not CBool(CryptExportKey(m_lHCryptKey, 0, PUBLICKEYBLOB, 0, m_sExportKey, lDataSize)) Then
        m_sErrorMsg = "Error during CryptExportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ExportPublicKey = True

    Exit Function

End Function

Public Function ImportPublicKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     ImportPublicKey
'*
'* Parameters:  None
'*
'* Purpose: This will import the public key of the other
'*          application, taking it from the ImportKey property.
'*          The imported public key will be held in the
'*          m_lHImportKey class variable.
'*************************************************************
On Error Resume Next
    ImportPublicKey = False

    Dim lDataSize As Long

    '--- Get the size of the key being imported
    lDataSize = Len(m_sImportKey)

    '--- Import the key, storing it in the m_lHImportKey class variable
    If Not CBool(CryptImportKey(m_lHCryptProv, m_sImportKey, lDataSize, 0, 0, m_lHImportKey)) Then
        m_sErrorMsg = "Error during CryptImportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ImportPublicKey = True

    Exit Function

End Function

Private Sub Class_Terminate()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       January 05, 1999
'*
'* Syntax:     Class_Terminate
'*
'* Parameters:  None
'*
'* Purpose: This is executed when this class is destroyed. This
'*          subroutine examines each of the class-level handles
'*          for the keys and keycontext, and destroys or
'*          releases them as is appropriate to clean up after
'*          oneself.
'*************************************************************
    Dim lResult As Long
    
    '--- Do we have an imported key? If so, destroy it.
    If (m_lHImportKey <> 0) Then lResult = CryptDestroyKey(m_lHImportKey)

    '--- Do we have a Public/Private keypair? If so, destroy it
    If (m_lHCryptKey <> 0) Then lResult = CryptDestroyKey(m_lHCryptKey)

    '--- Do we have an open key context? If so, release it.
    If (m_lHCryptProv <> 0) Then lResult = CryptReleaseContext(m_lHCryptProv, 0)
End Sub

Public Function GeneratePasswordKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     GeneratePasswordKey
'*
'* Parameters:  None
'*
'* Purpose: This will generate a session key from a hash of
'*          the password entered by the user. The session key
'*          will be placed into a private class variable.
'*************************************************************
On Error Resume Next
    Dim lHash As Long
    Dim lResult As Long
    
    GeneratePasswordKey = False
    
    '--- Create an empty hash object.
    If Not CBool(CryptCreateHash(m_lHCryptProv, CALG_MD5, 0, 0, lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptCreateHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Hash the password string.
    If Not CBool(CryptHashData(lHash, m_sPassword, Len(m_sPassword), 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptHashData!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Create a derived block cipher session key.
    If Not CBool(CryptDeriveKey(m_lHCryptProv, CALG_RC2, lHash, 0, m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDeriveKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Destroy the hash object
    If Not CBool(CryptDestroyHash(lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    GeneratePasswordKey = True
End Function

Public Property Get Password() As String
    Password = m_sPassword
End Property

Public Property Let Password(ByVal sNewValue As String)
    m_sPassword = sNewValue
End Property

Public Function EncryptMessageData()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     EncryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This function will take the data in the InBuffer property
'*          and encrypt it using the session key. The resulting
'*          encrypted data is placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    EncryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Determine the size of the buffer needed for encrypting the data
    '--- in the InBuffer property
    lCryptLength = Len(m_sInBuffer)
    lResult = CryptEncrypt(m_lHSessionKey, 0, 1, 0, vbNullString, lCryptLength, lCryptBufLen)
    
    '--- Prepare a string buffer for the CryptEncrypt function
    lCryptBufLen = lCryptLength * 2
    lCryptLength = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    
    '--- Copy in the contents of the InBuffer property
    LSet sCryptBuffer = m_sInBuffer

    '--- Encrypt data
    If Not CBool(CryptEncrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptLength, lCryptBufLen)) Then
        
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptEncrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If

    '--- Copy the encrypted data to the OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptLength)

    EncryptMessageData = True
End Function

Public Function DestroySessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DestroySessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will destroy the session key.
'*************************************************************
    Dim lResult As Long
    
    DestroySessionKey = False
    
    '--- Destroy the session key
    If Not CBool(CryptDestroyKey(m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDestroyKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    
    DestroySessionKey = True
End Function

Public Function DecryptMessageData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DecryptMessageData
'*
'* Parameters:  None
'*
'* Purpose: This will use the session key to decrypt the data in
'*          the InBuffer property. The decrypted data is then
'*          placed in the OutBuffer property.
'*************************************************************
On Error Resume Next
    DecryptMessageData = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Prepare sCryptBuffer for CryptDecrypt
    lCryptBufLen = Len(m_sInBuffer)
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    LSet sCryptBuffer = m_sInBuffer
    
    '--- Decrypt data
    If Not CBool(CryptDecrypt(m_lHSessionKey, 0, 1, 0, sCryptBuffer, lCryptBufLen)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptDecrypt!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        
        Exit Function
    End If
    
    '--- Apply decrypted string from sCryptBuffer to private buffer for OutBuffer property
    m_sOutBuffer = Mid$(sCryptBuffer, 1, lCryptBufLen)
    
    '--- Didn't exit early, return TRUE
    DecryptMessageData = True
End Function

Public Function HashFile() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     HashFile
'*
'* Parameters:  None
'*
'* Purpose: This function creates a hash value for the contents
'*          of the input buffer. The resulting hash value will
'*          be placed in the m_sHashBuffer variable for later
'*          use.
'*************************************************************
On Error Resume Next
    Dim lHash As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptBufLen As Long

    HashFile = False
    
    '--- Create an empty hash object.
    If Not CBool(CryptCreateHash(m_lHCryptProv, CALG_MD5, 0, 0, lHash)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptCreateHash!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Hash the password string.
    If Not CBool(CryptHashData(lHash, m_sInBuffer, Len(m_sInBuffer), 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptHashData!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Extract the hash value size.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHSIZE, sCryptBuffer, lCryptBufLen, 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGetHashParam!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Extract the hash value buffer size.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHVAL, vbNullString, lCryptBufLen, 0)) Then
        '--- Ignore, we expect this to give an error. But it does give us the size needed
        '--- for the hash buffer
    End If
    
    '--- Prepare sCryptBuffer for CryptDecrypt
    sCryptBuffer = String(lCryptBufLen, vbNullChar)
    
    '--- Extract the hash value.
    If Not CBool(CryptGetHashParam(lHash, HP_HASHVAL, sCryptBuffer, lCryptBufLen, 0)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGetHashParam!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If
    
    '--- Apply hash string from sCryptBuffer to private buffer for HashBuffer variables
    m_sHashBuffer = Mid$(sCryptBuffer, 1, lCryptBufLen)
    m_iHashLength = lCryptBufLen
    
    '--- Didn't exit early, return TRUE
    HashFile = True
End Function

Public Function EncryptFileData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     EncryptFileData
'*
'* Parameters:  None
'*
'* Purpose: This function places the hash of the original file
'*          at the front of the file (preceded by the hash
'*          length) and the calls EncryptMessageData to encrypt
'*          the whole thing.
'*************************************************************
    EncryptFileData = False
    
    Dim strInputData As String
    
    '--- Concatenate the length of the hash, the hash value, and the original
    '--- file contents
    strInputData = Trim(Str(m_iHashLength)) + m_sHashBuffer + m_sInBuffer
    
    '--- Copy the new string to the input buffer
    m_sInBuffer = strInputData
    
    '--- Call EncryptMessageData and return it's return value
    EncryptFileData = EncryptMessageData
End Function

Public Function DecryptFileData() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     DecryptFileData
'*
'* Parameters:  None
'*
'* Purpose: This function calls DecryptMessageData to decrypt
'*          the file contents. The original hash value is then
'*          stripped off the front of the decrypted data, and
'*          a new hash of the decrypted data is generated. If
'*          the two hash values match, the file has not been
'*          corrupted. If the two values don't match, the file
'*          has been corrupted.
'*************************************************************
    DecryptFileData = False
    
    Dim strInputData As String
    Dim strHash As String
    Dim strHashLength As String
    Dim lHashLength As Long
    
    '--- Decrypt the file data
    If Not DecryptMessageData Then Exit Function
    
    '--- Extract the original hash length
    strHashLength = Left(m_sOutBuffer, 2)
    lHashLength = CLng(strHashLength)
    
    '--- Extract the original hash value
    strHash = Mid(m_sOutBuffer, 3, lHashLength)
    
    '--- Extract the original file contents
    strInputData = Mid(m_sOutBuffer, (3 + lHashLength))
    
    '--- Copy the original file contents to the input buffer
    m_sInBuffer = strInputData
    
    '--- Hash the original file contents
    If Not HashFile Then Exit Function
        
    '--- Compare the original hash value to the new hash value
    If (strHash = m_sHashBuffer) Then
        '--- If they match, the file is unaltered, copy the original
        '--- file contents to the output buffer
        m_sOutBuffer = strInputData
    Else
        '--- The hash values don't match, the file has been corrupted
        MsgBox "File has been corrupted since original encryption."
        m_sOutBuffer = ""
        Exit Function
    End If
    
    '--- Didn't exit early, return TRUE
    DecryptFileData = True
End Function


Public Function GetExchangeKeypair() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 06, 1999
'*
'* Syntax:     GetExchangeKeypair
'*
'* Parameters:  None
'*
'* Purpose: This function loads the AT_KEYEXCHANGE keyset.
'*          If the keyset or key container does not exist,
'*          then this function creates them.
'*************************************************************
    Dim lResult As Long
    Dim sResult As String

    GetExchangeKeypair = False
    
    '--- Attempt to get handle to exchange key
    If Not CBool(CryptGetUserKey(m_lHCryptProv, AT_KEYEXCHANGE, m_lHCryptKey)) Then

        '--- Determine what the error was
        lResult = Err.LastDllError

        '--- If the problem is that there is no existing key, generate one
        If ((lResult = NTE_NO_KEY) Or (lResult = 0)) Then
            If Not CBool(CryptGenKey(m_lHCryptProv, AT_KEYEXCHANGE, 0, m_lHCryptKey)) Then
                m_sErrorMsg = "Error during CryptGenKey - " & CStr(Err.LastDllError)
                MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

                Exit Function
            End If
        Else

            '--- If there was another cause for the error, inform the user
            '--- what the problem is
            Select Case lResult

                Case ERROR_INVALID_HANDLE
                    sResult = "Invalid handle"

                Case ERROR_INVALID_PARAMETER
                    sResult = "Invalid parameter"

                Case NTE_BAD_KEY
                    sResult = "Bad key"

                Case NTE_BAD_UID
                    sResult = "Bad handle"
            End Select
            m_sErrorMsg = "Error during CryptGetUserKey - " + sResult
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

            Exit Function
        End If
    End If

    '--- Didn't exit early, return TRUE
    GetExchangeKeypair = True
End Function

Public Function CreateAndExportSessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 07, 1999
'*
'* Syntax:     CreateAndExportSessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will generate a session key and export it,
'*          encrypting the exported key using the public key
'*          of the recipient. The exported session key will
'*          be placed in the SessionKey property.
'*************************************************************
On Error Resume Next
    CreateAndExportSessionKey = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Create a random block cipher session key.
    If Not CBool(CryptGenKey(m_lHCryptProv, CALG_RC2, CRYPT_EXPORTABLE, m_lHSessionKey)) Then
        m_sErrorMsg = "Error " & CStr(Err.LastDllError) & " during CryptGenKey!"
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Determine the size of the key blob and allocate memory.
    lResult = CryptExportKey(m_lHSessionKey, m_lHImportKey, SIMPLEBLOB, 0, vbNullString, lDataSize)
    m_sSessionKey = String(lDataSize, vbNullChar)

    '--- Export the session key, using the imported public key to encrypt it
    If Not CBool(CryptExportKey(m_lHSessionKey, m_lHImportKey, SIMPLEBLOB, 0, m_sSessionKey, lDataSize)) Then
        m_sErrorMsg = "Error during CryptExportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    CreateAndExportSessionKey = True
End Function

Public Function ImportSessionKey() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       September 07, 1999
'*
'* Syntax:     ImportSessionKey
'*
'* Parameters:  None
'*
'* Purpose: This will import the session key from the SessionKey
'*          property.
'*************************************************************
On Error Resume Next
    ImportSessionKey = False

    Dim lDataSize As Long
    Dim lResult As Long
    Dim sCryptBuffer As String
    Dim lCryptLength As Long
    Dim lCryptBufLen As Long

    '--- Determine the size of the session key to be imported
    lDataSize = Len(m_sSessionKey)

    '--- Import the session key. As it should have been encrypted using
    '--- our public key, the CAPI should know to use our private key to
    '--- decrypt the session key during the import
    If Not CBool(CryptImportKey(m_lHCryptProv, m_sSessionKey, lDataSize, 0, 0, m_lHSessionKey)) Then
        m_sErrorMsg = "Error during CryptImportKey - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"

        Exit Function
    End If

    '--- Didn't exit early, return TRUE
    ImportSessionKey = True
End Function

Private Function GetCSPFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetCSPFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the CSP from the CRYPT_KEY_PROV_INFO
'*          type structure.
'*************************************************************
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    
    '--- Initialize the starting position
    iCurPos = 29
    '--- Trim off the CRYPT_KEY_PROV_INFO structure
    strChar = Mid(sbOrg, iCurPos)
    '--- Convert the string
    strRtn = StrConv(strChar, vbFromUnicode)
    '--- Find the end of the first string
    iCurPos = InStr(1, strRtn, vbNullChar, vbBinaryCompare)
    '--- Trim off the first string
    If iCurPos > 0 Then strRtn = Mid(strRtn, (iCurPos + 3))
    '--- Find the end of the second string
    iCurPos = InStr(1, strRtn, vbNullChar, vbBinaryCompare)
    '--- Trim the string one character shy of the NULL character
    If iCurPos > 0 Then strRtn = Left(strRtn, (iCurPos - 1))
    '--- Return the CSP name
    GetCSPFromKPI = strRtn
End Function

Private Function GetContainerNameFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetContainerNameFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the container name from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    
    '--- Extract the container name, starting at the
    '--- appropriate offset
    strRtn = ConvertBytesToString(Mid(sbOrg, 29))
    '--- Return the container name
    GetContainerNameFromKPI = strRtn
End Function

Private Function ConvertBytesToString(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     ConvertBytesToString(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will convert a byte array value to a string
'*          value by skipping every other character (byte) in
'*          the original string.
'*************************************************************
    Dim strNew As String
    Dim strCur As String
    Dim iCurPos As Integer
    
    '--- Convert the string from Unicode
    strNew = StrConv(sbOrg, vbFromUnicode)
    '--- Find any NULL characters in the string
    iCurPos = InStr(1, strNew, vbNullChar, vbBinaryCompare)
    '--- If any NULL characters were found, trim the string
    '--- one character shy of the NULL character
    If iCurPos > 0 Then strNew = Left(strNew, (iCurPos - 1))
    '--- Return the extracted string
    ConvertBytesToString = strNew
End Function

Public Function ListCerts(pszStoreName As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     ListCerts(pszStoreName)
'*
'* Parameters:  pszStoreName As String
'*
'* Purpose: This will loop through all of the certificates in
'*          the specified store, listing the available
'*          properties of each in turn. It builds this
'*          information as a large text string that is returned
'*          to the calling function to be displayed to the user.
'*************************************************************

    On Error GoTo ListCertsErr
    
    Dim hCertStore As Long
    Dim pszNameString As String * 256
    Dim pvData As String
    Dim dwData As Long
    Dim kcData As CERT_KEY_CONTEXT
    Dim kpiData As CRYPT_KEY_PROV_INFO
    Dim cbData As Long
    Dim dwPropId As Long
    Dim strMsg As String
    Dim lpCertContext As Long
    Dim szStoreName As String
    Dim szRtnMsg As String
    Dim iPos As Integer
    Dim iVerToUse As Integer
    
    szRtnMsg = ""
    dwPropId = 0

    '--- Open a system certificate store.
    hCertStore = CertOpenSystemStore(0, pszStoreName)
    
    '--- Did we open a store?
    If (hCertStore <> 0) Then
        szRtnMsg = "The " + pszStoreName + " store has been opened." + vbCrLf
    Else
        m_sErrorMsg = "Error during CertOpenSystemStore - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        Exit Function
    End If

    '--- Use CertEnumCertificatesInStore to get the certificates
    '--- from the open store.
    lpCertContext = CertEnumCertificatesInStore( _
        hCertStore, _
        0)

    '--- Loop until all certificates have been retrieved
    While (lpCertContext <> 0)
        
        '--- A certificate was retrieved. Get the name from the certificate.
        If (CertGetNameString( _
                lpCertContext, _
                CERT_NAME_SIMPLE_DISPLAY_TYPE, _
                0, _
                0, _
                pszNameString, _
                128)) Then
                
            '--- Find the end of the name
            iPos = InStr(1, pszNameString, vbNullChar, vbBinaryCompare)
            '--- Strip off the remaining NULL characters
            If iPos > 0 Then pszNameString = Left(pszNameString, (iPos - 1))
            '--- Tell the user the name of the certificate
            szRtnMsg = szRtnMsg + vbCrLf + "Certificate for " + Trim$(pszNameString) + vbCrLf
        Else
            m_sErrorMsg = "Error during CertGetNameString - " & CStr(Err.LastDllError) & Error$
            MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
        End If

        '--- Check validity of certificate
        If m_bUseCRLToVerifyCerts Then
            szRtnMsg = szRtnMsg + CheckCertAgainstCRL(lpCertContext) + vbCrLf
        Else
            szRtnMsg = szRtnMsg + CheckCertificateValidity(lpCertContext) + vbCrLf
        End If
        
        '--- Loop to find all of the property identifiers for the specified
        '--- certificate. The loop continues until
        '--- CertEnumCertificateContextProperties returns zero.

        dwPropId = CertEnumCertificateContextProperties( _
                lpCertContext, _
                dwPropId)
        While (dwPropId <> 0)
        
            '--- When the loop is executed, a property identifier has been found.
            '--- Print the property number.
            strMsg = "Property # " + Str(dwPropId) + " found -> "

            '--- Initialize the property data type indicator
            iVerToUse = 1
            
            '---  Indicate the kind of property found.
            Select Case dwPropId
                Case CERT_FRIENDLY_NAME_PROP_ID
                    strMsg = strMsg + "Friendly name: "
                    iVerToUse = 5
                Case CERT_SIGNATURE_HASH_PROP_ID
                    strMsg = strMsg + "Signature hash identifier "
                Case CERT_KEY_PROV_HANDLE_PROP_ID
                    strMsg = strMsg + "KEY PROVE HANDLE "
                    iVerToUse = 2
                Case CERT_KEY_PROV_INFO_PROP_ID
                    strMsg = strMsg + "KEY PROV INFO PROP ID "
                    iVerToUse = 4
                Case CERT_SHA1_HASH_PROP_ID
                    strMsg = strMsg + "SHA1 HASH identifier "
                Case CERT_MD5_HASH_PROP_ID
                    strMsg = strMsg + "md5 hash identifier "
                Case CERT_KEY_CONTEXT_PROP_ID
                    strMsg = strMsg + "KEY CONTEXT PROP identifier "
                Case CERT_KEY_SPEC_PROP_ID
                    strMsg = strMsg + "KEY SPEC PROP identifier "
                    iVerToUse = 2
                Case CERT_ENHKEY_USAGE_PROP_ID
                    strMsg = strMsg + "ENHKEY USAGE PROP identifier "
                Case CERT_NEXT_UPDATE_LOCATION_PROP_ID
                    strMsg = strMsg + "NEXT UPDATE LOCATION PROP identifier "
                Case CERT_PVK_FILE_PROP_ID
                    strMsg = strMsg + "PVK FILE PROP identifier "
                Case CERT_DESCRIPTION_PROP_ID
                    strMsg = strMsg + "DESCRIPTION PROP identifier "
                    iVerToUse = 5
                Case CERT_ACCESS_STATE_PROP_ID
                    strMsg = strMsg + "ACCESS STATE PROP identifier "
                    iVerToUse = 2
                Case CERT_SMART_CARD_DATA_PROP_ID
                    strMsg = strMsg + "SMART_CARD DATA PROP identifier "
                Case CERT_EFS_PROP_ID
                    strMsg = strMsg + "EFS PROP identifier "
                Case CERT_FORTEZZA_DATA_PROP_ID
                    strMsg = strMsg + "FORTEZZA DATA PROP identifier "
                Case CERT_ARCHIVED_PROP_ID
                    strMsg = strMsg + "ARCHIVED PROP identifier "
                    iVerToUse = 2
                Case CERT_KEY_IDENTIFIER_PROP_ID
                    strMsg = strMsg + "KEY IDENTIFIER PROP identifier "
                Case CERT_AUTO_ENROLL_PROP_ID
                    strMsg = strMsg + "AUTO ENROLL identifier. "
            End Select

            '--- Determine how to retrieve the property value based on the
            '--- property type
            Select Case iVerToUse
                Case 2
                    '--- Set the value size
                    cbData = 4
                    '--- Retrieve the property value
                    If (Not CBool(CertGetCertificateContextDWProperty( _
                            lpCertContext, _
                            dwPropId, _
                            dwData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextDWProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    strMsg = strMsg + vbCrLf + "The Property Content is " + Str(dwData)
                    
                Case 3
                    '--- Set the value size
                    cbData = 12
                    '--- Retrieve the property value
                    If (Not CBool(CertGetCertificateContextKCProperty( _
                            lpCertContext, _
                            dwPropId, _
                            kcData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextKCProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    '--- Show the results.
                    strMsg = strMsg + vbCrLf + "The Property Content is " + Str(kcData.dwKeySpec)
                
                Case 4
                    '--- Set the value size
                    cbData = 28
                    '--- Retrieve the property value
                    If (CBool(CertGetCertificateContextKPIProperty( _
                            lpCertContext, _
                            dwPropId, _
                            kpiData, _
                            cbData))) Then
                    End If
                    
                    '--- The call didn't succeed. Use the size to allocate memory
                    '--- for the property.
                    pvData = String(cbData, vbNullChar)
                    
                    '--- Allocation succeeded. Retrieve the property data.
                    If (Not CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            pvData, _
                            cbData))) Then
                        '--- Unable to retrieve the property value
                        m_sErrorMsg = "Error during CertGetCertificateContextProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    If cbData = 28 Then
                        '--- We retrieved the type structure
                        strMsg = strMsg + vbCrLf + "The Property Content is " + Str(kpiData.dwProvType)
                    Else
                        '--- We received a byte array
                        strMsg = strMsg + vbCrLf + "The Container Name is " + GetContainerNameFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The CSP is " + GetCSPFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The Provider Type is " + GetProvTypeFromKPI(pvData)
                        strMsg = strMsg + vbCrLf + "The Key Type is " + GetKeySpecFromKPI(pvData)
                    End If
                Case Else
                    '--- Retrieve information on the property by first getting the size
                    '--- of the property size.
                    cbData = 0
                    If CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            vbNullString, _
                            cbData)) Then
                    End If
                    
                    '--- The call succeeded. Use the size to allocate memory
                    '--- for the property.
                    pvData = String(cbData, vbNullChar)
                    
                    '--- Allocation succeeded. Retrieve the property data.
                    If (Not CBool(CertGetCertificateContextProperty( _
                            lpCertContext, _
                            dwPropId, _
                            pvData, _
                            cbData))) Then
                        m_sErrorMsg = "Error during CertGetCertificateContextProperty - " & CStr(Err.LastDllError) & Error$
                        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
                    End If
                    
                    '--- Show the results.
                    If iVerToUse = 5 Then
                        '--- The value is a string, extract it from the Byte array
                        strMsg = strMsg + vbCrLf + "The Property Content is " + ConvertBytesToString(pvData)
                    Else
                        '--- A byte array, display the ASCII value of the first character
                        strMsg = strMsg + vbCrLf + "The Property Content is " + Str(Asc(pvData))
                    End If
            End Select
            
            '--- Add the value to the return string
            szRtnMsg = szRtnMsg + strMsg + vbCrLf
            
            '--- Get the next property
            dwPropId = CertEnumCertificateContextProperties( _
                lpCertContext, _
                dwPropId)
        Wend
    
        '--- Get the next certificate
        lpCertContext = CertEnumCertificatesInStore( _
            hCertStore, _
            lpCertContext)
    Wend
    

    '--- Clean up.
    '--- If we have a current certificate, then release it
    If lpCertContext <> 0 Then
        CertFreeCertificateContext lpCertContext
    End If
    '--- Close the Certificate store
    CertCloseStore hCertStore, 0
    szRtnMsg = szRtnMsg + vbCrLf + "The function completed successfully."
    ListCerts = szRtnMsg
    
    Exit Function

ListCertsErr:
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function


Private Function GetProvTypeFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetProvTypeFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the provider type from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    Dim lProvType As Long
    
    '--- Extract the provider type, starting at the
    '--- appropriate offset
    lProvType = Asc(Mid(sbOrg, 12, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 11, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 10, 1))
    lProvType = (lProvType * 16) + Asc(Mid(sbOrg, 9, 1))
    '--- Determine the provider type
    Select Case lProvType
        Case PROV_RSA_FULL
            strRtn = "PROV_RSA_FULL"
        Case PROV_RSA_SIG
            strRtn = "PROV_RSA_SIG"
        Case PROV_DSS
            strRtn = "PROV_DSS"
        Case PROV_FORTEZZA
            strRtn = "PROV_FORTEZZA"
        Case PROV_MS_EXCHANGE
            strRtn = "PROV_MS_EXCHANGE"
        Case PROV_SSL
            strRtn = "PROV_SSL"
        Case PROV_RSA_SCHANNEL
            strRtn = "PROV_RSA_SCHANNEL"
        Case PROV_DSS_DH
            strRtn = "PROV_DSS_DH"
    End Select
    '--- Return the provider type
    GetProvTypeFromKPI = strRtn
End Function

Private Function GetKeySpecFromKPI(sbOrg As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       October 12, 1999
'*
'* Syntax:     GetKeySpecFromKPI(sbOrg)
'*
'* Parameters:  sbOrg As String
'*
'* Purpose: This will extract the key spec from the
'*          CRYPT_KEY_PROV_INFO type structure.
'*************************************************************
    Dim strCur As String
    Dim strChar As String
    Dim strRtn As String
    Dim iCurPos As Integer
    Dim iRelPos As Integer
    Dim lKeySpec As Long
    
    '--- Extract the key spec, starting at the
    '--- appropriate offset
    lKeySpec = Asc(Mid(sbOrg, 28, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 27, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 26, 1))
    lKeySpec = (lKeySpec * 16) + Asc(Mid(sbOrg, 25, 1))
    '--- Determine the key spec
    Select Case lKeySpec
        Case AT_KEYEXCHANGE
            strRtn = "AT_KEYEXCHANGE"
        Case AT_SIGNATURE
            strRtn = "AT_SIGNATURE"
    End Select
    '--- Return the key spec
    GetKeySpecFromKPI = strRtn
End Function

Public Property Get Server() As String
    Server = m_strServer
End Property

Public Property Let Server(ByVal sNewValue As String)
    m_strServer = sNewValue
End Property

Public Property Get CertificateAuthority() As String
    CertificateAuthority = m_strCA
End Property

Public Property Let CertificateAuthority(ByVal sNewValue As String)
    m_strCA = sNewValue
End Property

Public Function CheckCertificateValidity(lCertCtx As Long) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 7, 1999
'*
'* Syntax:     CheckCertificateValidity(lCertCtx)
'*
'* Parameters:  lCertCtx As Long
'*
'* Purpose: This will extract the serial number from the
'*          certificate information type structure. It then
'*          uses the Certificate Administrator object to check
'*          the certificate status with the CA.
'*************************************************************
    Dim cAdm As CCertAdmin
    Dim ccTx As CERT_CONTEXT
    Dim crtInfo As CERT_INFO
    Dim strSn As String
    Dim strIssuer As String
    Dim lRtn As Long
    Dim lRsn As Long
    Dim strMsg As String
    
    On Error GoTo CheckCertificateValidityErr
    
    '--- Initialize the return value
    CheckCertificateValidity = ""
    '--- Get the CERT_CONTEXT type structure
    CopyMemory ccTx, lCertCtx, 20
    '--- Get the CERT_INFO type structure
    CopyMemory crtInfo, ccTx.pCertInfo, 112
    '--- Size the serial number string to hold the certificate serial number
    strSn = String(crtInfo.SerialNumber.cbData, vbNullChar)
    '--- Copy the certificate serial number into the string
    CopyMemory ByVal strSn, crtInfo.SerialNumber.pbData, crtInfo.SerialNumber.cbData
    '--- Convert the serial number into a hex string
    strSn = ConvertBytesToHexString(strSn, crtInfo.SerialNumber.cbData)
    '--- Create the Cert Admin object
    Set cAdm = New CCertAdmin
    '--- Check the validity of the certificate
    lRtn = cAdm.IsValidCertificate(m_strServer + "\" + m_strCA, strSn)
    '--- Determine what the status of the certificate is
    Select Case lRtn
        Case CA_DISP_INCOMPLETE
            CheckCertificateValidity = "The Certificate Check was Incomplete"
        Case CA_DISP_ERROR
            CheckCertificateValidity = "The Certificate Check had an Error"
        Case CA_DISP_REVOKED
            strMsg = "Certificate has been Revoked"
            '--- The certificate has been revoked, get the reason
            lRsn = cAdm.GetRevocationReason
            Select Case lRsn
                Case CRL_REASON_UNSPECIFIED
                    strMsg = strMsg + vbCrLf + " --> Unspecified reason"
                Case CRL_REASON_KEY_COMPROMISE
                    strMsg = strMsg + vbCrLf + " --> Key was compromised"
                Case CRL_REASON_CA_COMPROMISE
                    strMsg = strMsg + vbCrLf + " --> Certificate Authority was compromised"
                Case CRL_REASON_AFFILIATION_CHANGED
                    strMsg = strMsg + vbCrLf + " --> Affiliation was changed"
                Case CRL_REASON_SUPERSEDED
                    strMsg = strMsg + vbCrLf + " --> Certificate was superseded"
                Case CRL_REASON_CESSATION_OF_OPERATION
                    strMsg = strMsg + vbCrLf + " --> Cessation of operation"
                Case CRL_REASON_CERTIFICATE_HOLD
                    strMsg = strMsg + vbCrLf + " --> Certificate placed on hold"
                Case CRL_REASON_REMOVE_FROM_CRL
                    strMsg = strMsg + vbCrLf + " --> Removed from CRL"
            End Select
            CheckCertificateValidity = strMsg
        Case CA_DISP_VALID
            CheckCertificateValidity = "Certificate is Valid"
        Case CA_DISP_INVALID
            CheckCertificateValidity = "Certificate is Invalid"
        Case CA_DISP_UNDER_SUBMISSION
            CheckCertificateValidity = "Certificate is Under Submission"
    End Select
    '--- Clean up
    Set cAdm = Nothing
    Exit Function
    
CheckCertificateValidityErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
    Resume Next
End Function

Private Function ConvertBytesToHexString(sbOrg As String, lbOrgLength As Long) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 7, 1999
'*
'* Syntax:     ConvertBytesToHexString(sbOrg, lbOrgLength)
'*
'* Parameters:  sbOrg As String
'*             lbOrgLength as Long
'*
'* Purpose: This will convert a byte string into a hex string.
'*************************************************************
    Dim strRtn As String
    Dim lPos As Long
    Dim strChr As String
    
    '--- Initialize the return string
    strRtn = ""
    '--- Loop through the bytes in reverse order
    For lPos = lbOrgLength To 1 Step -1
        '--- Get the current character's hex value
        strChr = Hex(Asc(Mid(sbOrg, lPos, 1)))
        '--- Do we need to pad it?
        If Len(strChr) < 2 Then
            '--- Pad the hex value so that it is 2 characters long
            If Len(strChr) = 1 Then
                strChr = "0" + strChr
            Else
                strChr = "00"
            End If
        End If
        '--- Append the hex value to the hex string
        strRtn = strRtn + strChr
    Next
    '--- Return the hex string
    ConvertBytesToHexString = strRtn
End Function

Public Sub GetCRLFromCA(pszStoreName As String, pszFileName As String)
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 7, 1999
'*
'* Syntax:     GetCRLFromCA(pszStoreName As String, pszFileName As String)
'*
'* Parameters:  pszStoreName As String
'*             pszFileName As String
'*
'* Purpose: This will read in a CRL list from a file retrieved
'*          from a CA and will load it into the certificate
'*          store.
'*************************************************************
    Dim hCertStore As Long
    Dim strCRL As String
    Dim strMsg As String
    Dim lCrlCtx As Long
    Dim lFileLength As Long
    Dim lFileNbr As Long
    
    '-- Get the length of the file to be decrypted
    lFileLength = FileLen(pszFileName)
    
    '-- Allocate the string to hold the entire file
    strCRL = String(lFileLength, vbNullChar)
    
    '-- Open the file in binary mode
    lFileNbr = FreeFile
    Open pszFileName For Binary Access Read As #lFileNbr
    
    '-- Read in the file
    Get #lFileNbr, , strCRL
    
    '-- Close the file
    Close #lFileNbr
    
    '--- Open a system certificate store.
    hCertStore = CertOpenSystemStore(0, pszStoreName)

    '--- Add the CRL to the certificate store
    If Not (CBool(CertAddEncodedCRLToStore(hCertStore, _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, _
            strCRL, Len(strCRL), _
            CERT_STORE_ADD_REPLACE_EXISTING_INHERIT_PROPERTIES, 0))) Then
            
        '--- Determine why the CRL couldn't be added
        Select Case Err.LastDllError
            Case CRYPT_E_EXISTS
                strMsg = "Certificate Revocation List already exists."
            Case CRYPT_E_OSS_ERROR
                strMsg = "ASN.1 Decoding Error."
            Case E_INVALIDARG
                strMsg = "Invalid argument."
            Case Else
                strMsg = "Unable to import the Certificate Revocation List"
        End Select
        '--- Display the error message for the user
        MsgBox strMsg
    End If
    
    '--- Close the Certificate store
    CertCloseStore hCertStore, 0
End Sub

Public Function CreateCRLAndCheckCerts(pszStoreName As String, pszFileName As String) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 7, 1999
'*
'* Syntax:     CreateCRLAndCheckCerts(pszStoreName As String,
'*                                      pszFileName As String)
'*
'* Parameters:  pszStoreName As String
'*             pszFileName As String
'*
'* Purpose: This will read in a CRL list from a file retrieved
'*          from a CA and create a CRL_CONTEXT type from it.
'*          It will then call the ListCerts function to list
'*          the certificates in the store, comparing them to
'*          the CRL to determine if they are valid.
'*************************************************************
    Dim strCRL As String
    Dim lFileLength As Long
    Dim lFileNbr As Long
    
    CreateCRLAndCheckCerts = ""
    '-- Get the length of the file to be decrypted
    lFileLength = FileLen(pszFileName)
    
    '-- Allocate the string to hold the entire file
    strCRL = String(lFileLength, vbNullChar)
    
    '-- Open the file in binary mode
    lFileNbr = FreeFile
    Open pszFileName For Binary Access Read As #lFileNbr
    
    '-- Read in the file
    Get #lFileNbr, , strCRL
    
    '-- Close the file
    Close #lFileNbr

    '--- Create the CRL from the file
    m_lCRLPointer = CertCreateCRLContext(X509_ASN_ENCODING Or _
            PKCS_7_ASN_ENCODING, strCRL, Len(strCRL))

    '--- Did we get a valid pointer?
    If m_lCRLPointer <> 0 Then
        '--- Specify that we'll be checking certs against the CRL
        m_bUseCRLToVerifyCerts = True
        '--- List the certs
        CreateCRLAndCheckCerts = ListCerts(pszStoreName)
        '--- Reset the flag
        m_bUseCRLToVerifyCerts = False
    End If
End Function

Private Function CheckCertAgainstCRL(lCertCtx As Long) As String
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 7, 1999
'*
'* Syntax:     CheckCertAgainstCRL(lCertCtx)
'*
'* Parameters:  lCertCtx As Long
'*
'* Purpose: This will compare the certificate context passed in
'*          to the CRL context currently held in memory to check
'*          whether the certificate is still valid.
'*************************************************************
    Dim ccTx As CERT_CONTEXT
    Dim cCrlTx As CRL_CONTEXT
    Dim strMsg As String
    Dim larrCrlInfos(1) As Long
    
    On Error GoTo CheckCertAgainstCRLErr
    
    '--- Initialize the return value
    CheckCertAgainstCRL = ""
    '--- Get the CERT_CONTEXT type structure
    CopyMemory ccTx, lCertCtx, 20
    '--- Get the CRL_CONTEXT type structure
    CopyMemory cCrlTx, m_lCRLPointer, 20
    '--- Put the pointer to the CRL_INFO type into an array
    larrCrlInfos(0) = cCrlTx.pCrlInfo
    '--- Check the validity of the certificate
    If CBool(CertVerifyCRLRevocation( _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, _
            ccTx.pCertInfo, 1, larrCrlInfos(0))) Then
        strMsg = "Certificate is Valid"
    Else
        strMsg = "Certificate is NOT Valid!!!"
    End If
    '--- Return the results
    CheckCertAgainstCRL = strMsg
    Exit Function
    
CheckCertAgainstCRLErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
    Resume Next
End Function

Public Function GetSignatureCertificate(pszStoreName As String, pszCertName As String) As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     GetSignatureCertificate(pszStoreName, pszCertName)
'*
'* Parameters:  pszStoreName As String
'*             pszCertName As String
'*
'* Purpose: This function loops through the certificates in the
'*          specified certificate store, until the signature
'*          certificate for the specified user is found. This
'*          function returns a pointer to the first signature
'*          certificate found for the specified user, and doesn't
'*          perform any validation checking to determine if the
'*          certificate has either expired or been revoked.
'*************************************************************
    Dim bContinue As Boolean
    Dim lCertCtxPtr As Long
    Dim hCertStore As Long
    
    On Error GoTo GetSignatureCertificateErr
    
    '--- Initialize the return value
    GetSignatureCertificate = False
    bContinue = True
    
    '--- Open a system certificate store.
    m_hCertStore = CertOpenSystemStore(0, pszStoreName)
    '--- If unable to open the specified store, exit
    If m_hCertStore = 0 Then Exit Function
    '--- Find the first certificate in the store for the
    '--- specified user.
    lCertCtxPtr = CertFindCertificateInStore(m_hCertStore, _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, 0, _
            CERT_FIND_SUBJECT_STR, pszCertName, 0)
    '--- If no certificate found, exit
    If lCertCtxPtr = 0 Then Exit Function
    '--- Do we have a signature certificate? If so, then
    '--- set the flag to indicate that we don't need to
    '--- continue past this point.
    If GetKeyType(lCertCtxPtr) = AT_SIGNATURE Then bContinue = False
    '--- Loop until a signature certificate is found, or
    '--- there are no more certificates to be returned
    While bContinue
        '--- Get the next certificate for the specified user.
        lCertCtxPtr = CertFindCertificateInStore(m_hCertStore, _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, 0, _
            CERT_FIND_SUBJECT_STR, pszCertName, lCertCtxPtr)
        '--- If no more certificates, end processing
        If lCertCtxPtr = 0 Then
            bContinue = False
        Else
            '--- If a signature certificate found, end processing
            If GetKeyType(lCertCtxPtr) = AT_SIGNATURE Then bContinue = False
        End If
    Wend
    '--- Set the certificate context pointer for the class signature
    '--- certificate variable.
    m_lCertContextPointer = lCertCtxPtr
    '--- Did we find a signature certificate, change the
    '--- return value to TRUE.
    If lCertCtxPtr <> 0 Then GetSignatureCertificate = True
    '--- Exit the function
    Exit Function

GetSignatureCertificateErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function

Public Function SignMessage() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     SignMessage
'*
'* Parameters:  None
'*
'* Purpose: This function uses the CryptSignMessage function
'*          to sign the message stored in the InBuffer property
'*          using the signature certificate held in the class
'*          variable, and to place the signed message in the
'*          OutBuffer property.
'*************************************************************
    Dim csmpSignParams As CRYPT_SIGN_MESSAGE_PARA
    Dim strSignedMsg As String
    Dim lMsgSize As Long
    Dim lRtnVal As Long
    Dim lCertArray(1) As Long
    Dim lMsgArray(1) As String
    Dim lMsgLenArray(1) As Long
    
    On Error GoTo SignMessageErr
    
    '--- Initialize the return value
    SignMessage = False
    '--- Fill in the signature parameter type structure
    '--- The type structure size
    csmpSignParams.cbSize = 68
    '--- The encoding type
    csmpSignParams.dwMsgEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The pointer to the certificate context of the certificate
    '--- to be used for signing the message
    csmpSignParams.pSigningCert = m_lCertContextPointer
    '--- The hash algorith to be used for signing
    csmpSignParams.HashAlgorithm.pszObjId = szOID_RSA_MD5
    '--- Any auxiliary information? No
    csmpSignParams.pvHashAuxInfo = 0
    '--- Number of certificates to be included with the
    '--- signed message.
    csmpSignParams.cMsgCert = 1
    '--- Pointers to the certificate contexts of the certificates
    '--- to be included in the signed message.
    lCertArray(0) = m_lCertContextPointer
    csmpSignParams.rgpMsgCert = VarPtr(lCertArray(0))
    '--- Nothing else
    csmpSignParams.cMsgCrl = 0
    csmpSignParams.cAuthAttr = 0
    csmpSignParams.cUnauthAttr = 0
    csmpSignParams.dwFlags = 0
    csmpSignParams.dwInnerContentType = 0
    '--- Copy the message to the array of messages to be signed
    lMsgArray(0) = m_sInBuffer
    '-- And the length of the message
    lMsgLenArray(0) = Len(m_sInBuffer)
    '--- Initialize the signed message buffer
    strSignedMsg = vbNullChar
    '--- Call CryptSignMessage to get the required size of the
    '--- signed message buffer.
    lRtnVal = CryptSignMessage(csmpSignParams, 0, 1, lMsgArray(0), lMsgLenArray(0), _
            strSignedMsg, lMsgSize)
    '--- Did we get a buffer size?
    If (lMsgSize > 0) Then
        '--- Size the buffer
        strSignedMsg = String(lMsgSize, vbNullChar)
        '--- Sign the message
        If Not CBool(CryptSignMessage(csmpSignParams, 0, 1, lMsgArray(0), lMsgLenArray(0), _
                strSignedMsg, lMsgSize)) Then
            '--- If something failed, then exit the function
            Exit Function
        End If
    Else
        '--- No message buffer size, something must have gone wrong.
        Exit Function
    End If
    '--- Copy the signed message to the output buffer.
    m_sOutBuffer = strSignedMsg
    '--- Update the return value.
    SignMessage = True
    '--- Exit the function
    Exit Function
    
SignMessageErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function

Public Function ValidateSignature() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     ValidateSignature
'*
'* Parameters:  None
'*
'* Purpose: This function uses the CryptValidateSignature
'*          function to validate the signature on the message.
'*          It also extracts the message from the encoded signed
'*          message.
'*************************************************************
    Dim cvmpVerifyParams As CRYPT_VERIFY_MESSAGE_PARA
    Dim strDecodedMsg As String
    Dim lMsgLength As Long
    
    On Error GoTo ValidateSignatureErr
    
    '--- Initialize the return value
    ValidateSignature = False
    '--- Fill in the signature validation parameter type structure
    '--- The type structure size
    cvmpVerifyParams.cbSize = 20
    '--- The encoding type
    cvmpVerifyParams.dwMsgAndCertEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The CSP handle
    cvmpVerifyParams.hCryptProv = m_lHCryptProv
    '--- Get the length of the signed message
    lMsgLength = Len(m_sInBuffer)
    '--- Allocate the buffer to received the extracted message.
    strDecodedMsg = String(lMsgLength, vbNullChar)
    '--- Verify the message signature
    If CBool(CryptVerifyMessageSignature(cvmpVerifyParams, 0, m_sInBuffer, lMsgLength, _
            strDecodedMsg, lMsgLength, 0)) Then
        '--- The signature was valid, set the return value
        ValidateSignature = True
        '--- Copy the extracted message to the output buffer
        m_sOutBuffer = Left(strDecodedMsg, lMsgLength)
    End If
    '--- Exit the function
    Exit Function
    
ValidateSignatureErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function

Public Function SignAndEncryptMessage(strName As String) As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     SignAndEncryptMessage(strName)
'*
'* Parameters:  strName As String
'*
'* Purpose: This function uses the CryptSignAndEncryptMessage
'*          function to perform both the signing and encrypting
'*          of the message in a single step.
'*************************************************************
    Dim csmpSignParams As CRYPT_SIGN_MESSAGE_PARA
    Dim cempEncryptParams As CRYPT_ENCRYPT_MESSAGE_PARA
    Dim strSignedMsg As String
    Dim lMsgSize As Long
    Dim lRtnVal As Long
    Dim lCertArray(1) As Long
    Dim lRecpCertArray(1) As Long
    Dim lMsgArray(1) As String
    Dim lMsgLenArray(1) As Long

    On Error GoTo SignAndEncryptMessageErr
    
    '--- Initialize the return value
    SignAndEncryptMessage = False
    '--- Fill in the signature parameter type structure
    '--- The type structure size
    csmpSignParams.cbSize = 68
    '--- The encoding type
    csmpSignParams.dwMsgEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The pointer to the certificate context of the certificate
    '--- to be used for signing the message
    csmpSignParams.pSigningCert = m_lCertContextPointer
    '--- The hash algorithm to be used for signing
    csmpSignParams.HashAlgorithm.pszObjId = szOID_RSA_MD5
    '--- Any auxiliary information? No
    csmpSignParams.pvHashAuxInfo = 0
    '--- Number of certificates to be included with the
    '--- signed message.
    csmpSignParams.cMsgCert = 1
    '--- Pointers to the certificate contexts of the certificates
    '--- to be included in the signed message.
    lCertArray(0) = m_lCertContextPointer
    csmpSignParams.rgpMsgCert = VarPtr(lCertArray(0))
    '--- Nothing else
    csmpSignParams.cMsgCrl = 0
    csmpSignParams.cAuthAttr = 0
    csmpSignParams.cUnauthAttr = 0
    csmpSignParams.dwFlags = 0
    csmpSignParams.dwInnerContentType = 0
    '--- Fill in the encryption parameter type structure
    '--- The type structure size
    cempEncryptParams.cbSize = 32
    '--- The encoding type
    cempEncryptParams.dwMsgEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The handle of the CSP
    cempEncryptParams.hCryptProv = m_lHCryptProv
    '--- The encryption algorithm to use
    cempEncryptParams.ContentEncryptionAlgorithm.pszObjId = szOID_RSA_RC4
    '--- Get a pointer to the certificate context of the key
    '--- exchange certificate of the recipient of the message.
    lRecpCertArray(0) = GetRecipientCertificate(strName)
    '--- Initialize the signed message buffer
    strSignedMsg = vbNullChar
    lMsgSize = 0
    '--- Call CryptSignAndEncryptMessage to get the required size of the
    '--- signed message buffer.
    lRtnVal = CryptSignAndEncryptMessage(csmpSignParams, cempEncryptParams, 1, _
        lRecpCertArray(0), m_sInBuffer, Len(m_sInBuffer), strSignedMsg, lMsgSize)
    '--- Did we get a buffer size?
    If (lMsgSize > 0) Then
        '--- Size the buffer
        strSignedMsg = String(lMsgSize, vbNullChar)
        '--- Sign and encrypt the message
        If Not CBool(CryptSignAndEncryptMessage(csmpSignParams, cempEncryptParams, 1, _
                lRecpCertArray(0), m_sInBuffer, Len(m_sInBuffer), strSignedMsg, lMsgSize)) Then
            '--- If something failed, then exit the function
            Exit Function
        End If
    Else
        '--- No message buffer size, something must have gone wrong.
        Exit Function
    End If
    '--- Copy the signed and encrypted message to the output buffer
    m_sOutBuffer = strSignedMsg
    '--- Update the return value
    SignAndEncryptMessage = True
    '--- Release the recipient certificate
    CertFreeCertificateContext lRecpCertArray(0)
    '--- Exit the function
    Exit Function
    
SignAndEncryptMessageErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function

Public Function DecryptAndValidateMessage() As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     DecryptAndValidateMessage
'*
'* Parameters:  None
'*
'* Purpose: This function decrypts and verifies the signature
'*          on a message using the CryptDecryptAndVerifyMessage
'*          function. This performs both the decryption and
'*          signature verification in a single step.
'*************************************************************
    Dim cdmpDecryptParams As CRYPT_DECRYPT_MESSAGE_PARA
    Dim cvmpVerifyParams As CRYPT_VERIFY_MESSAGE_PARA
    Dim strDecodedMsg As String
    Dim lMsgLength As Long
    Dim lStoreArray(1) As Long

    On Error GoTo DecryptAndValidateMessageErr
    '--- Initialize the return value
    DecryptAndValidateMessage = False
    '--- Fill in the signature validation parameter type structure
    '--- The type structure size
    cvmpVerifyParams.cbSize = 20
    '--- The encoding type
    cvmpVerifyParams.dwMsgAndCertEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The handle of the CSP
    cvmpVerifyParams.hCryptProv = m_lHCryptProv
    '--- Fill in the decryption parameter type structure
    '--- The type structure size
    cdmpDecryptParams.cbSize = 16
    '--- The encoding type
    cdmpDecryptParams.dwMsgAndCertEncodingType = X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING
    '--- The number of certificate store handles being passed
    cdmpDecryptParams.cCertStore = 1
    '--- Handles to the certificate stores to be searched for the decryption
    '--- certificate and associated private key.
    lStoreArray(0) = m_hCertStore
    cdmpDecryptParams.rghCertStore = VarPtr(lStoreArray(0))
    '--- Get the message length
    lMsgLength = Len(m_sInBuffer)
    '--- Allocate the buffer for the decrypted message
    strDecodedMsg = String(lMsgLength, vbNullChar)
    '--- Decrypt and verify the message
    If CBool(CryptDecryptAndVerifyMessageSignature(cdmpDecryptParams, cvmpVerifyParams, _
            0, m_sInBuffer, lMsgLength, _
            strDecodedMsg, lMsgLength, 0, 0)) Then
        '--- The signature was valid, set the return value
        DecryptAndValidateMessage = True
        '--- Copy the extracted message to the output buffer
        m_sOutBuffer = Left(strDecodedMsg, lMsgLength)
    End If
    '--- Exit the function
    Exit Function
    
DecryptAndValidateMessageErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function

Private Function GetCertName(lpCertCtx As Long) As String
'******* NOT CURRENTLY USED !!!!! ****************************
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     GetCertName(lpCertCtx)
'*
'* Parameters:  lpCertCtx As Long
'*             pszCertName As String
'*
'* Purpose: This function extracts the subject name from the
'*          certificate passed in.
'*************************************************************
    Dim pszNameString As String * 256
    Dim iPos As Integer
    
    '--- A certificate was retrieved. Get the name from the certificate.
    If (CertGetNameString( _
            lpCertCtx, _
            CERT_NAME_SIMPLE_DISPLAY_TYPE, _
            0, _
            0, _
            pszNameString, _
            128)) Then
            
        '--- Find the end of the name
        iPos = InStr(1, pszNameString, vbNullChar, vbBinaryCompare)
        '--- Strip off the remaining NULL characters
        If iPos > 0 Then pszNameString = Left(pszNameString, (iPos - 1))
    Else
        '--- Set the name to a blank string
        pszNameString = ""
    End If
    '--- Return the name string
    GetCertName = pszNameString
End Function

Public Sub ReleaseSignatureCertificate()
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     ReleaseSignatureCertificate
'*
'* Parameters:  None
'*
'* Purpose: This subroutine releases certificate context of the
'*          signature certificate and closes the certificate
'*          store.
'*************************************************************
    '--- If we have a current certificate, then release it
    If m_lCertContextPointer <> 0 Then
        CertFreeCertificateContext m_lCertContextPointer
    End If
    '--- If we have a certificate store handle, then close it
    If m_hCertStore <> 0 Then
        '--- Close the Certificate store
        CertCloseStore m_hCertStore, 0
    End If
End Sub

Private Function GetKeyType(lpCertCtx As Long) As Long
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     GetKeyType(lpCertCtx)
'*
'* Parameters:  lpCertCtx As Long
'*
'* Purpose: This function extracts the key type from the
'*          certificate context that is passed in.
'*************************************************************
    Dim kpiData As CRYPT_KEY_PROV_INFO
    Dim dwData As Long
    Dim pvData As String
    Dim cbData As Long
                    
    '--- Set the value size
    cbData = 28
    '--- Retrieve the property value
    If (CBool(CertGetCertificateContextKPIProperty( _
            lpCertCtx, _
            CERT_KEY_PROV_INFO_PROP_ID, _
            kpiData, _
            cbData))) Then
    End If
    
    '--- The call didn't succeed. Use the size to allocate memory
    '--- for the property.
    pvData = String(cbData, vbNullChar)
    
    '--- Allocation succeeded. Retrieve the property data.
    If (Not CBool(CertGetCertificateContextProperty( _
            lpCertCtx, _
            CERT_KEY_PROV_INFO_PROP_ID, _
            pvData, _
            cbData))) Then
        '--- Unable to retrieve the property value
        m_sErrorMsg = "Error during CertGetCertificateContextProperty - " & CStr(Err.LastDllError) & Error$
        MsgBox m_sErrorMsg, vbOKOnly, "VB Crypto"
    End If
    
    '--- Show the results.
    If cbData = 28 Then
        '--- We retrieved the type structure
        GetKeyType = kpiData.dwKeySpec
    Else
        '--- We received a byte array
        If GetKeySpecFromKPI(pvData) = "AT_SIGNATURE" Then
            GetKeyType = AT_SIGNATURE
        Else
            GetKeyType = AT_KEYEXCHANGE
        End If
    End If

End Function

Public Function GetRecipientCertificate(pszCertName As String) As Long
'*************************************************************
'* Written By: Davis Chapman
'* Date:       November 29, 1999
'*
'* Syntax:     GetRecipientCertificate(pszCertName)
'*
'* Parameters:  pszCertName As String
'*
'* Purpose: This function retrieves the key exchange certificate
'*          for the user specified from the open certificate
'*          store.
'*************************************************************
    Dim bContinue As Boolean
    Dim lCertCtxPtr As Long
    Dim hCertStore As Long
    
    On Error GoTo GetRecipientCertificateErr
    '--- Initialize the return value
    GetRecipientCertificate = 0
    bContinue = True
    '--- If the certificate store is not already open, exit
    If m_hCertStore = 0 Then Exit Function
    '--- Find the first certificate in the store for the
    '--- specified user.
    lCertCtxPtr = CertFindCertificateInStore(m_hCertStore, _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, 0, _
            CERT_FIND_SUBJECT_STR, pszCertName, 0)
    '--- If no certificate found, exit
    If lCertCtxPtr = 0 Then Exit Function
    '--- Do we have a key exchange certificate? If so, then
    '--- set the flag to indicate that we don't need to
    '--- continue past this point.
    If GetKeyType(lCertCtxPtr) = AT_KEYEXCHANGE Then bContinue = False
    '--- Loop until a key exchange certificate is found, or
    '--- there are no more certificates to be returned
    While bContinue
        '--- Get the next certificate for the specified user.
        lCertCtxPtr = CertFindCertificateInStore(m_hCertStore, _
            X509_ASN_ENCODING Or PKCS_7_ASN_ENCODING, 0, _
            CERT_FIND_SUBJECT_STR, pszCertName, lCertCtxPtr)
        '--- If no more certificates, end processing
        If lCertCtxPtr = 0 Then bContinue = False
        '--- If a key exchange certificate found, end processing
        If GetKeyType(lCertCtxPtr) = AT_KEYEXCHANGE Then bContinue = False
    Wend
    '--- Did we find a key exchange certificate, change the
    '--- return value to the pointer to the certificate context.
    If lCertCtxPtr <> 0 Then GetRecipientCertificate = lCertCtxPtr
    '--- Exit the function
    Exit Function

GetRecipientCertificateErr:
    '--- An error occurred, display it for the user
    MsgBox "Error: " + Str(Err.Number) + " - " + Err.Description
End Function


