VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 1  'NoTransaction
END
Attribute VB_Name = "clsLoginDomain"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
'*************************************************************
'* Written By: Davis Chapman
'* Date:       April 7, 1999
'*
'* Purpose: This DLL provides a server-size object that can be
'*          called to validate a user name/password combination
'*          for the specified NT domain. It returns a boolean
'*          value in a recordset that specifies whether the
'*          login is valid. An invalid indicator can also signal
'*          a server-side error in the validation process that
'*          prevented this object from validating the ID/Password.
'*
'* Revisions
'*   Name          Date     Description
'*   ---------------------------------------------------------
'*************************************************************
Option Explicit

'--- Declare constants for use in API calls
Const ANYSIZE_ARRAY = 1

'--- Error Codes
Const ERROR_SUCCESS = 0&
Const ERROR_NO_TOKEN = 1008&
Const ERROR_NOT_ALL_ASSIGNED = 1300&
Const ERROR_LOGON_FAILURE = 1326&
'--- Security Privileges to be adjusted
Const SE_TCB_NAME = "SeTcbPrivilege"
'--- Security check types and providers
Const LOGON32_LOGON_BATCH = 4
Const LOGON32_LOGON_INTERACTIVE = 2
Const LOGON32_LOGON_SERVICE = 5
Const LOGON32_LOGON_NETWORK = 3
Const LOGON32_PROVIDER_DEFAULT = 0
Const LOGON32_PROVIDER_WINNT35 = 1
Const LOGON32_PROVIDER_WINNT40 = 2
Const LOGON32_PROVIDER_WINNT50 = 3
'--- Token constants
Const TokenPrivileges = 3
Const TokenDefaultDacl = 6
Const TokenGroups = 2
Const TokenImpersonationLevel = 9
Const TokenOwner = 4
Const TokenPrimaryGroup = 5
Const TokenSource = 7
Const TokenStatistics = 10
Const TokenType = 8
Const TokenUser = 1
'--- Permission constants
Const DELETE = &H10000
Const READ_CONTROL = &H20000
Const WRITE_DAC = &H40000
Const WRITE_OWNER = &H80000
Const SYNCHRONIZE = &H100000
Const STANDARD_RIGHTS_REQUIRED = &HF0000
Const STANDARD_RIGHTS_READ = READ_CONTROL
Const STANDARD_RIGHTS_WRITE = READ_CONTROL
Const STANDARD_RIGHTS_EXECUTE = READ_CONTROL
Const STANDARD_RIGHTS_ALL = &H1F0000
'--- Token permission constants
Const TOKEN_ASSIGN_PRIMARY = &H1
Const TOKEN_DUPLICATE = &H2
Const TOKEN_IMPERSONATE = &H4
Const TOKEN_QUERY = &H8
Const TOKEN_QUERY_SOURCE = &H10
Const TOKEN_ADJUST_PRIVILEGES = &H20
Const TOKEN_ADJUST_GROUPS = &H40
Const TOKEN_ADJUST_DEFAULT = &H80
Const TOKEN_ADJUST_SESSIONID = &H100

Const TOKEN_ALL_ACCESS = (STANDARD_RIGHTS_REQUIRED Or _
                          TOKEN_ASSIGN_PRIMARY Or _
                          TOKEN_DUPLICATE Or _
                          TOKEN_IMPERSONATE Or _
                          TOKEN_QUERY Or _
                          TOKEN_QUERY_SOURCE Or _
                          TOKEN_ADJUST_PRIVILEGES Or _
                          TOKEN_ADJUST_GROUPS Or _
                          TOKEN_ADJUST_SESSIONID Or _
                          TOKEN_ADJUST_DEFAULT)

Const TOKEN_READ = (STANDARD_RIGHTS_READ Or _
                          TOKEN_QUERY)

Const TOKEN_WRITE = (STANDARD_RIGHTS_WRITE Or _
                          TOKEN_ADJUST_PRIVILEGES Or _
                          TOKEN_ADJUST_GROUPS Or _
                          TOKEN_ADJUST_DEFAULT)

Const TOKEN_EXECUTE = (STANDARD_RIGHTS_EXECUTE)

Const TOKEN_MY_ACCESS = (TOKEN_READ Or TOKEN_WRITE)

'--- Enable privilege constant
Const SE_PRIVILEGE_ENABLED = &H2

'--- Security Privilege types and structures
Private Type luid
    LowPart As Long
    HighPart As Long
End Type

Private Type LUID_AND_ATTRIBUTES
        pLuid As luid
        Attributes As Long
End Type

Private Type TOKEN_PRIVILEGES
    PrivilegeCount As Long
    Privileges(ANYSIZE_ARRAY) As LUID_AND_ATTRIBUTES
End Type

'--- Security and privilege API declarations
Private Declare Function LogonUser Lib "advapi32" Alias "LogonUserA" ( _
    ByVal lpszUsername As String, _
    ByVal lpszDomain As String, _
    ByVal lpszPassword As String, _
    ByVal dwLogonType As Long, _
    ByVal dwLogonProvider As Long, _
    phToken As Long) As Long

Private Declare Function CloseHandle Lib "kernel32" ( _
    ByVal hObject As Long) As Long

Private Declare Function OpenThreadToken Lib "advapi32.dll" ( _
    ByVal ThreadHandle As Long, _
    ByVal DesiredAccess As Long, _
    ByVal OpenAsSelf As Long, _
    TokenHandle As Long) As Long
    
Private Declare Function OpenProcessToken Lib "advapi32.dll" ( _
    ByVal ProcessHandle As Long, _
    ByVal DesiredAccess As Long, _
    TokenHandle As Long) As Long
    
Private Declare Function GetCurrentProcess Lib "kernel32" () As Long

Private Declare Function GetCurrentThread Lib "kernel32" () As Long

Private Declare Function LookupPrivilegeValue Lib "advapi32.dll" Alias "LookupPrivilegeValueA" ( _
    ByVal lpSystemName As String, _
    ByVal lpName As String, _
    lpLuid As luid) As Long
    
Private Declare Function AdjustTokenPrivileges Lib "advapi32.dll" ( _
    ByVal TokenHandle As Long, _
    ByVal DisableAllPrivileges As Long, _
    NewState As TOKEN_PRIVILEGES, _
    ByVal BufferLength As Long, _
    PreviousState As Long, _
    ReturnLength As Long) As Long

Public Function LoginUser(sUserName As String, sDomain As String, sPassword As String) As Long
'*************************************************************
'* Written By: Davis Chapman
'* Date:       April 7, 1999
'*
'* Syntax:     LoginUser(sUserName, sDomain, sPassword)
'*
'* Parameters: sUserName As String - The logon name to be validated
'*             sDomain As String - The domain to validate the logon name in
'*             sPassword As String - The password for the logon name
'*
'* Purpose: This method validates the user name/password
'*          combination in the NT domain specified. It
'*          opens a context token to be used in setting the
'*          privileges so that the logon can be validated (as
'*          well as setting the privileges back). Finally, it
'*          calls the LogonUser API call to validate the logon.
'*          If the logon is successful, the session token is
'*          returned to the calling process.
'*************************************************************
    Dim lToken As Long
    Dim lHandle As Long
    Dim lRtnVal As Long
    Dim lType As Long
    Dim lProv As Long
    Dim dwErr As Long
On Error GoTo CheckLogin_Err

    '--- Set the default return value to the initialized recordset
    LoginUser = 0
    '--- Set the logon type and provider to be used
    lType = LOGON32_LOGON_INTERACTIVE
    lProv = LOGON32_PROVIDER_DEFAULT
    '--- Get a thread or process token that can be used for
    '--- setting the privileges so that the logon can be validated
    lRtnVal = OpenThreadToken(GetCurrentThread, TOKEN_MY_ACCESS, 0, lHandle)
    '--- Were we successful?
    If (lRtnVal = 0) Then
        '--- No, check the error
        dwErr = Err.LastDllError
        '--- Did we not get a token?
        If (dwErr = ERROR_NO_TOKEN) Then
            '--- Try to get a process token
            lRtnVal = OpenProcessToken(GetCurrentProcess, TOKEN_MY_ACCESS, lHandle)
            '--- Were we successful?
            If (lRtnVal = 0) Then
                '--- No, check the error
                dwErr = Err.LastDllError
                '--- Log the error and exit the function
                App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
                Exit Function
            End If
        Else
            '--- Log the error and exit the function
            App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
            Exit Function
        End If
    End If
    '--- Set the necessary privilege
    If (Not SetPrivilege(lHandle, SE_TCB_NAME, True)) Then Exit Function
    '--- Validate the user logon
    lRtnVal = LogonUser(sUserName, sDomain, sPassword, lType, lProv, lToken)
    '--- Were we successful?
    If lRtnVal <> 0 Then
        '--- Non-zero, the call worked. Check the error status to see
        '--- if there were problems
        dwErr = Err.LastDllError
        Select Case dwErr
            '--- Invalid logon combination
            Case ERROR_LOGON_FAILURE
                '--- Do nothing
                'App.LogEvent "Login validation error = ERROR_LOGON_FAILURE", vbLogEventTypeError
            '--- No error, valid logon
            Case 0
                '--- Reset the return value to return the logon token
                LoginUser = lToken
            '--- Some other error occurred
            Case Else
                '--- Log the error
                App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
        End Select
    Else
        '--- Zero, the call failed. Check the error status to see
        '--- what problems were
        lRtnVal = Err.LastDllError
        '--- Invalid logon combination?
        If lRtnVal <> ERROR_LOGON_FAILURE Then
            '--- No, log the error
            App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
        Else
            '--- Yes, do nothing
            'App.LogEvent "Login validation error = ERROR_LOGON_FAILURE", vbLogEventTypeError
        End If
    End If
    '--- Reset the necessary privilege
    If (Not SetPrivilege(lHandle, SE_TCB_NAME, False)) Then Exit Function
    '--- Close the thread or process handle that is used to set the privilege
    lRtnVal = CloseHandle(lHandle)
    '--- Exit the function
    Exit Function
    
CheckLogin_Err:
    '--- An error occurred, log the error
    App.LogEvent "Logon validation error = " & Err.Number & " = " & Err.Description
    '--- Exit the function
    Exit Function
End Function

Private Function SetPrivilege(hToken As Long, strPrivilege As String, bEnablePriv As Boolean) As Boolean
'*************************************************************
'* Written By: Davis Chapman
'* Date:       April 7, 1999
'*
'* Syntax:     SetPrivilege(hToken, strPrivilege, bEnablePriv)
'*
'* Parameters: hToken As Long - The token of the current process/thread
'*             strPrivilege As String - The privilege to be modified
'*             bEnablePriv As Boolean - Enable or disable the privilege
'*
'* Purpose: This method is called to modify the privileges of
'*          the current process/thread so that restricted API
'*          functions can be called.
'*************************************************************
On Error GoTo SetPrivilege_Err
    Dim tp As TOKEN_PRIVILEGES
    Dim lLuid As luid
    Dim dwErr As Long
    Dim lRtnVal As Long
    
    '--- Initialize the return value
    SetPrivilege = False
    '--- Lookup the LUID of the requested privilege
    lRtnVal = LookupPrivilegeValue(vbNullString, strPrivilege, lLuid)
    '--- Were we successful?
    If lRtnVal = 0 Then
        '--- No, get the error ID and log the error
        dwErr = Err.LastDllError
        App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
        Exit Function
    End If
    '--- Initialize the token privileges structure with the LUID
    tp.PrivilegeCount = 1
    tp.Privileges(0).pLuid.LowPart = lLuid.LowPart
    tp.Privileges(0).pLuid.HighPart = lLuid.HighPart
    '--- Specify whether to enable or disable the specified privilege
    If bEnablePriv Then
        tp.Privileges(0).Attributes = SE_PRIVILEGE_ENABLED
    Else
        tp.Privileges(0).Attributes = 0
    End If
    
    '--- Enable or disable the privilege
    AdjustTokenPrivileges hToken, 0, tp, 16, 0, 0
    '--- Get the error code to determine if we were successful
    dwErr = Err.LastDllError
    '--- Did we have an error occur?
    If (dwErr <> ERROR_SUCCESS) And (dwErr <> ERROR_NOT_ALL_ASSIGNED) Then
        '--- Yes, log the error and exit
        App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
        Exit Function
    Else
        '--- Possibly not, check to see if there might have been a problem
        If (dwErr = ERROR_NOT_ALL_ASSIGNED) Then
            '--- Log the error but continue on
            App.LogEvent "Login validation error = " & dwErr, vbLogEventTypeError
        End If
    End If
    '--- Return TRUE
    SetPrivilege = True
    '--- Exit the function
    Exit Function
    
SetPrivilege_Err:
    '--- An error occurred, log the error
    App.LogEvent "Login validation error = " & Err.Number & " - " & Err.Description
    '--- Exit the function
    Exit Function
End Function

Public Sub CloseLogin(lToken As Long)
    Dim lRtnVal As Long
    
    '--- Close the token created
    lRtnVal = CloseHandle(lToken)
End Sub
